import new_plot
pyPlot = new_plot
from Numeric import *
from scipy_base.scimath import *


def _intialize():
	global _figure; global _active
	_figure = [pyPlot.Plot()]
	_active = _figure[-1]

_intialize()

def _validate_active():
	global _figure; global _active
	if _active.valid(): return
	try: _figure.remove(_active)
	except ValueError: pass
	try: 
		_active = _figure[-1]		
		_validate_active()
	except IndexError: _intialize()
	
def figure(which_one = None):
	global _figure; global _active
	if which_one is None:
		_figure.append(pyPlot.Plot())
		_active = _figure[-1]
	else:
		try:	_figure.index(which_one)
		except ValueError: _figure.append(which_one)
		_active = which_one

def current():
	_validate_active()
	return _active
	
def close(which_one = None):
	global _figure; global _active
	if which_one is None:
		_active.close()
		try: _active = _figure[-1]
		except IndexError: _intialize()
	elif which_one == 'all':
		for fig in _figure: fig.close()
		_intialize()
	else:
		try: 
			_figure.remove(which_one)
			which_one.close()
		except ValueError:
			which_one.close() 
	if _active == which_one: 
		try: _active = _figure[-1]
		except IndexError: _intialize()

def cmp_imag(x,y):
	if x.real == y.real and x.imag == y.imag: return 0
	if x.real <= y.real: return -1
	if x.real>= y.real: return 1

def zplane(data,title=None, style = 19):
	data = asarray(data,Complex)
	xmin = floor(min(data.real))
	xmax = ceil(max(data.real))
	ymin = floor(min(data.imag))
	ymax = ceil(max(data.imag))
	if(ymax - ymin) == 0:
		ymax = xmax
		ymin = xmin
	if(xmax - xmin) == 0:
		xmax = ymax
		xmin = ymin
	if xmax  < 1.: xmax = 1.
	if ymax  < 1.: ymax = 1.
	if xmin  > -1.: xmin = -1.	
	if ymin  > -1.: ymin = -1.	
	t = arange(100)*2*pi/100.-pi
	_active.plot(sin(t),cos(t),'notitle w l lt 0')
	_active.hold('on')
	_active.plot([xmin,xmax],[0.,0.],'notitle w l lt 0')
	_active.plot([0.,0.],[ymin,ymax],'notitle w l lt 0')
	if title:
		_active.plot(data.real,data.imag, 'title "%s"w p pt %d ps 3' % (title,style))
	else:	
		_active.plot(data.real,data.imag, 'notitle w p pt %d ps 3' % (title,style))
	rr = array(data).tolist()
	rr.sort(cmp_imag)
	SMALL = 1e-2
	for i in rr:
		count = 0
		for j in rr:
			if abs(i-j) < SMALL: count = count + 1
		#print i.real, i.imag, count	
		if count > 1:
			xpos = 1.03 *(( i.real - xmin) / (xmax - xmin))
			ypos = 1.03 *((i.imag - ymin) / (ymax - ymin))
			_active.text((xpos,ypos),"%d"%count)	
	_active.xaxis([xmin,xmax])
	_active.yaxis([ymin,ymax])
	_active._send('set size ratio %f' % ((ymax-ymin)/(xmax-xmin)))	
	_active.hold('off')
	_active.grid('off')
def plot(*data):
	_validate_active()
	apply(_active.plot,data)
def polar(*data):
	_validate_active()
	apply(_active.polar,data)
def autoscale():
	_validate_active()
	_active.autoscale()
def logx(st='on'):
	_validate_active()
	_active.logx(st)
def logy(st='on'):
	_validate_active()
	_active.logy(st)	
def xaxis(rng):
	_validate_active()
	_active.xaxis(rng)
def yaxis(rng):
	_validate_active()
	_active.yaxis(rng)
def zaxis(rng):
	_validate_active()
	_active.zaxis(rng)
def raxis(rng):
	_validate_active()
	_active.raxis(rng)
def taxis(rng):
	_validate_active()
	_active.taxis(rng)
def grid(st):
	_validate_active()
	_active.grid(st)
def hold(st):
	_validate_active()
	_active.hold(st)
def title(t):
	_validate_active()
	_active.title(t)
def xtitle(t):
	_validate_active()
	_active.xtitle(t)
def ytitle(t):
	_validate_active()
	_active.ytitle(t)
def ztitle(t):
	_validate_active()
	_active.ztitle(t)	
def text(xyz,*t):
	_validate_active()
	apply(_active.text,(xyz,)+t )
def label_coord(c):
	_validate_active()
	_active.label_coord(c)
def legend(cmd=''):
	_validate_active()
	_active.legend(cmd)
#3d plotting stuff		
def surf(*data):
	_validate_active()
	apply(_active.surf,data)
def plot3d(*data):
	_validate_active()
	apply(_active.plot3d,data)
def mapping(val):
	_validate_active()
	_active.mapping(val)
def angles(val):
	_validate_active()
	_active.angles(val)
def view(val):
	_validate_active()
	_active.view(val)
def ticlevel(val):
	_validate_active()
	_active.ticlevel(val)
def hidden(val):
	_validate_active()
	_active.hidden(val)
def output(filename,type,options = ''):
	_validate_active()
	_active.output(filename,type,options)

def png(filename):
	_validate_active()
	_active.png(filename)
def save(filename):
	_validate_active()
	_active.png(filename)

def size(size=(1.,1.)):
	_validate_active()
	_active.size(size)




#!/usr/bin/env python

import sys
from scipy_test.testing import *
set_package_path()
from optimize import zeros as cc
del sys.path[0]

from math import sin,sqrt,log
from random import random

def f1(x) :
    return x*(x-1.)

def f2(x) :
    return x**2 - 1

def f3(x) :
    return x*(x-1.)*(x-2.)*(x-3.)

def f4(x) :
    if x > 1 : return  1.0 + .1*x
    if x < 1 : return -1.0 + .1*x
    return 0

def f5(x) :
    if x != 1 : return 1.0/(1. - x)
    return 0

def f6(x) :
    if   x > 1 : return random()
    elif x < 1 : return -random()
    else : return 0

description =\
"""
f2 is a symmetric parabola, x**2 - 1
f3 is a quartic polynomial with large hump in interval
f4 is step function with a discontinuity at 1
f5 is a hyperbola with vertical asymptote at 1
f6 has random values positive to left of 1 , negative to right

of course these are not real problems. They just test how the
'good' solvers behave in bad circumstances where bisection is
really the best. A good solver should not be much worse than
bisection in such circumstance, while being faster for smooth
monotone sorts of functions.
"""

methods = [cc.bisect,cc.ridder,cc.brenth,cc.brentq]
mstrings = ['cc.bisect','cc.ridder','cc.brenth','cc.brentq']
functions = [f2,f3,f4,f5,f6]
fstrings = ['f2','f3','f4','f5','f6']

from scipy_test.testing import ScipyTestCase
import unittest


class test_basic(ScipyTestCase) :

    def check_run(self) :
        a = .5
        b = sqrt(3)
        repeat = 2000

        print 'TESTING CONVERGENCE\n'
        print 'zero should be 1\n'
        for i in range(len(functions)) :
            print 'function %s\n'%fstrings[i]
            for j in range(len(methods)) :
                try:
                    zero = methods[j](functions[i],a,b)
                except:
                    print '%s : failed'%mstrings[j]
                else:
                    print '%s : %21.16f'%(mstrings[j],zero)
            print '\n\n'

    def bench_run(self,level=5):
        a = .5
        b = sqrt(3)
        repeat = 2000

        print '%s\n',description

        print 'TESTING SPEED\n'
        print 'times in seconds for %d iterations \n'%repeat
        for i in range(len(functions)) :
            print 'function %s\n'%fstrings[i]
            func = functions[i]
            for j in range(len(methods)) :
                meth = methods[j]
                try:
                    t = self.measure("meth(func,a,b)",repeat)
                except:
                    print '%s : failed'%mstrings[j]
                else:
                    print '%s : %5.3f'%(mstrings[j],t)
            print '\n\n'

if __name__ == '__main__' :
    ScipyTest('optimize.zeros').run()



# should re-write compiled functions to take a local and global dict
# as input.
import sys,os
import ext_tools
import string
import catalog
import common_info

# not an easy way for the user_path_list to come in here.
# the PYTHONCOMPILED environment variable offers the most hope.

function_catalog = catalog.catalog()


class inline_ext_function(ext_tools.ext_function):
    # Some specialization is needed for inline extension functions
    def function_declaration_code(self):
       code  = 'static PyObject* %s(PyObject*self, PyObject* args)\n{\n'
       return code % self.name

    def template_declaration_code(self):
        code = 'template<class T>\n' \
               'static PyObject* %s(PyObject*self, PyObject* args)\n{\n'
        return code % self.name

    def parse_tuple_code(self):
        """ Create code block for PyArg_ParseTuple.  Variable declarations
            for all PyObjects are done also.

            This code got a lot uglier when I added local_dict...
        """
        declare_return = 'py::object return_val;\n'    \
                         'int exception_occured = 0;\n'    \
                         'PyObject *py__locals = NULL;\n' \
                         'PyObject *py__globals = NULL;\n'

        py_objects = ', '.join(self.arg_specs.py_pointers())
        if py_objects:
            declare_py_objects = 'PyObject ' + py_objects +';\n'
        else:
            declare_py_objects = ''

        py_vars = ' = '.join(self.arg_specs.py_variables())
        if py_vars:
            init_values = py_vars + ' = NULL;\n\n'
        else:
            init_values = ''

        parse_tuple = 'if(!PyArg_ParseTuple(args,"OO:compiled_func",'\
                                           '&py__locals,'\
                                           '&py__globals))\n'\
                      '    return NULL;\n'

        return   declare_return + declare_py_objects + \
                 init_values + parse_tuple

    def arg_declaration_code(self):
        arg_strings = []
        for arg in self.arg_specs:
            arg_strings.append(arg.declaration_code(inline=1))
        code = string.join(arg_strings,"")
        return code

    def arg_cleanup_code(self):
        arg_strings = []
        for arg in self.arg_specs:
            arg_strings.append(arg.cleanup_code())
        code = string.join(arg_strings,"")
        return code

    def arg_local_dict_code(self):
        arg_strings = []
        for arg in self.arg_specs:
            arg_strings.append(arg.local_dict_code())
        code = string.join(arg_strings,"")
        return code


    def function_code(self):
        from ext_tools import indent
        decl_code = indent(self.arg_declaration_code(),4)
        cleanup_code = indent(self.arg_cleanup_code(),4)
        function_code = indent(self.code_block,4)
        #local_dict_code = indent(self.arg_local_dict_code(),4)

        try_code =    'try                              \n' \
                      '{                                \n' \
                      '    PyObject* raw_locals = py_to_raw_dict('       \
                                             'py__locals,"_locals");\n'  \
                      '    PyObject* raw_globals = py_to_raw_dict('      \
                                          'py__globals,"_globals");\n' + \
                      '    /* argument conversion code */     \n' + \
                           decl_code                               + \
                      '    /* inline code */                   \n' + \
                           function_code                           + \
                      '    /*I would like to fill in changed    '   \
                              'locals and globals here...*/   \n'   \
                      '\n}                                       \n'
        catch_code =  "catch(...)                        \n"   \
                      "{                                 \n" + \
                      "    return_val =  py::object();   \n"   \
                      "    exception_occured = 1;        \n"   \
                      "}                                 \n"   
        return_code = "    /* cleanup code */                   \n" + \
                           cleanup_code                             + \
                      "    if(!(PyObject*)return_val && !exception_occured)\n"   \
                      "    {\n                                  \n"   \
                      "        return_val = Py_None;            \n"   \
                      "    }\n                                  \n"   \
                      "    return return_val.disown();          \n"           \
                      "}                                \n"

        all_code = self.function_declaration_code()         + \
                       indent(self.parse_tuple_code(),4)    + \
                       indent(try_code,4)                   + \
                       indent(catch_code,4)                 + \
                       return_code

        return all_code

    def python_function_definition_code(self):
        args = (self.name, self.name)
        function_decls = '{"%s",(PyCFunction)%s , METH_VARARGS},\n' % args
        return function_decls

class inline_ext_module(ext_tools.ext_module):
    def __init__(self,name,compiler=''):
        ext_tools.ext_module.__init__(self,name,compiler)
        self._build_information.append(common_info.inline_info())

function_cache = {}
def inline(code,arg_names=[],local_dict = None, global_dict = None,
           force = 0,
           compiler='',
           verbose = 0,
           support_code = None,
           headers = [],
           customize=None,
           type_converters = None,
           auto_downcast=1,
           **kw):
    """ Inline C/C++ code within Python scripts.

        inline() compiles and executes C/C++ code on the fly.  Variables
        in the local and global Python scope are also available in the
        C/C++ code.  Values are passed to the C/C++ code by assignment
        much like variables passed are passed into a standard Python
        function.  Values are returned from the C/C++ code through a
        special argument called return_val.  Also, the contents of
        mutable objects can be changed within the C/C++ code and the
        changes remain after the C code exits and returns to Python.

        inline has quite a few options as listed below.  Also, the keyword
        arguments for distutils extension modules are accepted to
        specify extra information needed for compiling.

        code -- string. A string of valid C++ code.  It should not specify a
                return statement.  Instead it should assign results that
                need to be returned to Python in the return_val.
        arg_names -- optional. list of strings. A list of Python variable names 
                     that should be transferred from Python into the C/C++ 
                     code.  It defaults to an empty string.
        local_dict -- optional. dictionary. If specified, it is a dictionary
                      of values that should be used as the local scope for the
                      C/C++ code.  If local_dict is not specified the local
                      dictionary of the calling function is used.
        global_dict -- optional. dictionary.  If specified, it is a dictionary
                       of values that should be used as the global scope for
                       the C/C++ code.  If global_dict is not specified the
                       global dictionary of the calling function is used.
        force --      optional. 0 or 1. default 0.  If 1, the C++ code is
                      compiled every time inline is called.  This is really
                      only useful for debugging, and probably only useful if
                      your editing support_code a lot.
        compiler --   optional. string.  The name of compiler to use when
                      compiling.  On windows, it understands 'msvc' and 'gcc'
                      as well as all the compiler names understood by
                      distutils.  On Unix, it'll only understand the values
                      understood by distutils. ( I should add 'gcc' though
                      to this).

                      On windows, the compiler defaults to the Microsoft C++
                      compiler.  If this isn't available, it looks for mingw32
                      (the gcc compiler).

                      On Unix, it'll probably use the same compiler that was
                      used when compiling Python. Cygwin's behavior should be
                      similar.
        verbose --    optional. 0,1, or 2. defualt 0.  Speficies how much
                      much information is printed during the compile phase
                      of inlining code.  0 is silent (except on windows with
                      msvc where it still prints some garbage). 1 informs
                      you when compiling starts, finishes, and how long it
                      took.  2 prints out the command lines for the compilation
                      process and can be useful if your having problems
                      getting code to work.  Its handy for finding the name
                      of the .cpp file if you need to examine it.  verbose has
                      no affect if the compilation isn't necessary.
        support_code -- optional. string.  A string of valid C++ code declaring
                        extra code that might be needed by your compiled
                        function.  This could be declarations of functions,
                        classes, or structures.
        headers      -- optional. list of strings.  A list of strings specifying
                        header files to use when compiling the code.  The list 
                        might look like ["<vector>","'my_header'"].  Note that 
                        the header strings need to be in a form than can be 
                        pasted at the end of a #include statement in the 
                        C++ code.
        customize --   optional. base_info.custom_info object. An alternative
                       way to specify support_code, headers, etc. needed by
                       the function see the compiler.base_info module for more
                       details. (not sure this'll be used much).
        type_converters -- optional. list of type converters. These
                          guys are what convert Python data types to C/C++ data
                          types.  If you'd like to use a different set of type
                          conversions than the default, specify them here. Look
                          in the type conversions section of the main
                          documentation for examples.
        auto_downcast -- optional. 0 or 1. default 1.  This only affects
                         functions that have Numeric arrays as input variables.
                         Setting this to 1 will cause all floating point values
                         to be cast as float instead of double if all the
                         Numeric arrays are of type float.  If even one of the
                         arrays has type double or double complex, all
                         variables maintain there standard types.

        Distutils keywords.  These are cut and pasted from Greg Ward's
        distutils.extension.Extension class for convenience:

        sources : [string]
          list of source filenames, relative to the distribution root
          (where the setup script lives), in Unix form (slash-separated)
          for portability.  Source files may be C, C++, SWIG (.i),
          platform-specific resource files, or whatever else is recognized
          by the "build_ext" command as source for a Python extension.
          Note: The module_path file is always appended to the front of this
                list
        include_dirs : [string]
          list of directories to search for C/C++ header files (in Unix
          form for portability)
        define_macros : [(name : string, value : string|None)]
          list of macros to define; each macro is defined using a 2-tuple,
          where 'value' is either the string to define it to or None to
          define it without a particular value (equivalent of "#define
          FOO" in source or -DFOO on Unix C compiler command line)
        undef_macros : [string]
          list of macros to undefine explicitly
        library_dirs : [string]
          list of directories to search for C/C++ libraries at link time
        libraries : [string]
          list of library names (not filenames or paths) to link against
        runtime_library_dirs : [string]
          list of directories to search for C/C++ libraries at run time
          (for shared extensions, this is when the extension is loaded)
        extra_objects : [string]
          list of extra files to link with (eg. object files not implied
          by 'sources', static library that must be explicitly specified,
          binary resource files, etc.)
        extra_compile_args : [string]
          any extra platform- and compiler-specific information to use
          when compiling the source files in 'sources'.  For platforms and
          compilers where "command line" makes sense, this is typically a
          list of command-line arguments, but for other platforms it could
          be anything.
        extra_link_args : [string]
          any extra platform- and compiler-specific information to use
          when linking object files together to create the extension (or
          to create a new static Python interpreter).  Similar
          interpretation as for 'extra_compile_args'.
        export_symbols : [string]
          list of symbols to be exported from a shared extension.  Not
          used on all platforms, and not generally necessary for Python
          extensions, which typically export exactly one symbol: "init" +
          extension_name.
    """
    # this grabs the local variables from the *previous* call
    # frame -- that is the locals from the function that called
    # inline.
    global function_catalog

    call_frame = sys._getframe().f_back
    if local_dict is None:
        local_dict = call_frame.f_locals
    if global_dict is None:
        global_dict = call_frame.f_globals
    if force:
        module_dir = global_dict.get('__file__',None)
        func = compile_function(code,arg_names,local_dict,
                                global_dict,module_dir,
                                compiler=compiler,
                                verbose=verbose,
                                support_code = support_code,
                                headers = headers,
                                customize=customize,
                                type_converters = type_converters,
                                auto_downcast = auto_downcast,
                                **kw)

        function_catalog.add_function(code,func,module_dir)
        results = attempt_function_call(code,local_dict,global_dict)
    else:
        # 1. try local cache
        try:
            results = apply(function_cache[code],(local_dict,global_dict))
            return results
        except TypeError, msg: 
            msg = str(msg).strip()
            if msg[:16] == "Conversion Error":
                pass
            else:
                raise TypeError, msg
        except NameError, msg: 
            msg = str(msg).strip()
            if msg[:16] == "Conversion Error":
                pass
            else:
                raise NameError, msg
        except KeyError:
            pass
        # 2. try function catalog
        try:
            results = attempt_function_call(code,local_dict,global_dict)
        # 3. build the function
        except ValueError:
            # compile the library
            module_dir = global_dict.get('__file__',None)
            func = compile_function(code,arg_names,local_dict,
                                    global_dict,module_dir,
                                    compiler=compiler,
                                    verbose=verbose,
                                    support_code = support_code,
                                    headers = headers,
                                    customize=customize,
                                    type_converters = type_converters,
                                    auto_downcast = auto_downcast,
                                    **kw)

            function_catalog.add_function(code,func,module_dir)
            results = attempt_function_call(code,local_dict,global_dict)
    return results

def attempt_function_call(code,local_dict,global_dict):
    # we try 3 levels here -- a local cache first, then the
    # catalog cache, and then persistent catalog.
    #
    global function_cache
    # 2. try catalog cache.
    function_list = function_catalog.get_functions_fast(code)
    for func in function_list:
        try:
            results = apply(func,(local_dict,global_dict))
            function_catalog.fast_cache(code,func)
            function_cache[code] = func
            return results
        except TypeError, msg: # should specify argument types here.
            # This should really have its own error type, instead of
            # checking the beginning of the message, but I don't know
            # how to define that yet.
            msg = str(msg)
            if msg[:16] == "Conversion Error":
                pass
            else:
                raise TypeError, msg
        except NameError, msg: 
            msg = str(msg).strip()
            if msg[:16] == "Conversion Error":
                pass
            else:
                raise NameError, msg                
    # 3. try persistent catalog
    module_dir = global_dict.get('__file__',None)
    function_list = function_catalog.get_functions(code,module_dir)
    for func in function_list:
        try:
            results = apply(func,(local_dict,global_dict))
            function_catalog.fast_cache(code,func)
            function_cache[code] = func
            return results
        except: # should specify argument types here.
            pass
    # if we get here, the function wasn't found
    raise ValueError, 'function with correct signature not found'

def inline_function_code(code,arg_names,local_dict=None,
                         global_dict=None,auto_downcast = 1,
                         type_converters=None,compiler=''):
    call_frame = sys._getframe().f_back
    if local_dict is None:
        local_dict = call_frame.f_locals
    if global_dict is None:
        global_dict = call_frame.f_globals
    ext_func = inline_ext_function('compiled_func',code,arg_names,
                                   local_dict,global_dict,auto_downcast,
                                   type_converters = type_converters)
    import build_tools
    compiler = build_tools.choose_compiler(compiler)
    ext_func.set_compiler(compiler)
    return ext_func.function_code()

def compile_function(code,arg_names,local_dict,global_dict,
                     module_dir,
                     compiler='',
                     verbose = 0,
                     support_code = None,
                     headers = [],
                     customize = None,
                     type_converters = None,
                     auto_downcast=1,
                     **kw):
    # figure out where to store and what to name the extension module
    # that will contain the function.
    #storage_dir = catalog.intermediate_dir()
    module_path = function_catalog.unique_module_name(code,module_dir)
    storage_dir, module_name = os.path.split(module_path)
    mod = inline_ext_module(module_name,compiler)

    # create the function.  This relies on the auto_downcast and
    # type factories setting
    ext_func = inline_ext_function('compiled_func',code,arg_names,
                                   local_dict,global_dict,auto_downcast,
                                   type_converters = type_converters)
    mod.add_function(ext_func)

    # if customize (a custom_info object), then set the module customization.
    if customize:
        mod.customize = customize

    # add the extra "support code" needed by the function to the module.
    if support_code:
        mod.customize.add_support_code(support_code)
    
    # add the extra headers needed by the function to the module.
    for header in headers:
        mod.customize.add_header(header)
        
    # compile code in correct location, with the given compiler and verbosity
    # setting.  All input keywords are passed through to distutils
    mod.compile(location=storage_dir,compiler=compiler,
                verbose=verbose, **kw)

    # import the module and return the function.  Make sure
    # the directory where it lives is in the python path.
    try:
        sys.path.insert(0,storage_dir)
        exec 'import ' + module_name
        func = eval(module_name+'.compiled_func')
    finally:
        del sys.path[0]
    return func




""" Testing gui_thread.  Please run this under gui_thread and also
without it.  Right now the testing is extremely rudimentary.  The
proxied return types are tested so are a few simple classes.

To test the code you should run the tests both without and with using
gui_thread in another thread.  Like so:

 $ python
 >>> import test_gui_thread
 >>> test_gui_thread.test()
 # or
 $ python test_gui_thread.py

And then:
 $ python
 >>> import gui_thread
 >>> <Importing wxPython...  done.>
 >>> import test_gui_thread
 
 >>> test_gui_thread.test()

"""

from wxPython.wx import *

import unittest, gui_thread, weakref, time, sys, math

def threaded():
    return gui_thread.main.running_in_second_thread

def is_alive(obj):
    if obj() is None:
        return 0
    else:
        return 1

def Yield():
    if not threaded():
        # forces the event handlers to finish their work.
        # this also forces deletion of windows.
        wxYield() 
    else:
        time.sleep(0.05) # sync threads
    

class TestPanel(wxPanel):
    def __init__(self, parent):
        wxPanel.__init__(self, parent, -1)
        self.attr = None
        ID = NewId()
        btn = wxButton(self, ID, "Hello")
        EVT_BUTTON(self, ID, self.OnButton)

    def OnButton(self, evt):
        print "TestPanel.OnButton"


class TestFrame(wxFrame):
    def __init__(self, parent):
        wxFrame.__init__(self, parent, -1, "Hello Test")
        self.panel = TestPanel(self)
        self.attr = [0, 0]
        EVT_CLOSE(self, self.OnClose)
        self.Show(1)

    def OnClose(self, evt):
        self.Destroy()


class TestClass:
    def __init__(self):
        self.a = 1

    def test(self):
        return self.a

class _TestHang:
    def get_obj(self):
        val = TestHang()
        return val

TestHang = gui_thread.register(_TestHang)

class _A:
    pass

A = gui_thread.register(_A)

class _TestDeref:
    def __init__(self):
        self.a = A()
        
    def test_proxy(self, arg1):
        "Returns 0 if the test fails."
        if (gui_thread.main.is_proxy(arg1) == 1) or \
           (gui_thread.main.is_proxy_attr(arg1) == 1):
            return 0
        else:
            return 1

    def test_proxy_attr(self):
        a = TestDeref()
        return self.test_proxy(a.a)

    def test_kw(self, arg1=None):
        return self.test_proxy(arg1)

TestDeref = gui_thread.register(_TestDeref)

dummy_instance = TestClass()

class TestProxyAttr:
    def __init__(self):
        self.lst = [1,2]
        self.dict = {'a':'a', 'b':'b'}

    def test(self):
        return 1

    def get_int(self):
        return 1

    def get_float(self):
        return math.pi

    def get_complex(self):
        return 1.0 + 5j

    def get_string(self):
        return "This is a string"

    def get_list(self):
        return [[0,1,2, 'foo', dummy_instance], 1,2, "foo", dummy_instance]

    def get_tuple(self):
        return ((1,2,3), 1, 2, 'foo',  dummy_instance)

    def get_dict(self):
        dict = {'int': 1, 'string': "This is a string",
                'list': [1,2,'foo'], 'tuple':(1,2, 'foo'),
                'obj': dummy_instance,
                'dict': {'int':1, 'string': 'This is a string'}}
        return dict

    def get_callable(self):
        return self.test

class test_gui_thread(unittest.TestCase):
    def check_wx_class(self):
        "Checking a wxFrame proxied class"
        for i in range(5):
            f = gui_thread.register(TestFrame)
            a = f(None)
            p = weakref.ref(a)
            a.Close(1)
            del a
            Yield()
            # this checks for memory leaks
            self.assertEqual(is_alive(p), 0)
            
    def check_normal_class(self):
        "Checking non-wxWindows proxied class "
        f = gui_thread.register(TestClass)
        a = f()
        p = weakref.ref(a)
        # the reference count has to be 2 since nothing special should
        # be done for these classes.
        self.assertEqual(sys.getrefcount(a), 2)
        del a
        self.assertEqual(is_alive(p), 0)        
        
    def check_exception(self):
        "Checking exception handling"
        f = gui_thread.register(TestFrame)
        a = f(None)
        p = weakref.ref(a)
        self.assertRaises(TypeError, a.Close, 1, 2, 3)
        a.Close()
        del a
        Yield()
        # this checks for memory leaks
        self.assertEqual(is_alive(p), 0)

    def check_proxy_proxy(self):
        "Checking proxied function calling another proxied function"
        a = TestHang()
        a.get_obj()

    def check_dereference(self):
        "Checking if dereferencing works."
        a = TestDeref()
        self.assertEqual(a.test_proxy(a), 1)
        self.assertEqual(a.test_kw(arg1=a), 1)
        self.assertEqual(a.test_proxy_attr(), 1)

    def check_setattr(self):
        "Checking if __setattr__ works."
        f = gui_thread.register(TestFrame)
        a = f(None)
        a.attr = [1, "string"]
        self.assertEqual(a.attr[0], 1)
        self.assertEqual(a.attr[1], "string")

        # checking the proxy_attr's __setattr__
        a.panel.attr = [1, "string"]
        self.assertEqual(a.panel.attr[0], 1)
        self.assertEqual(a.panel.attr[1], "string")
        a.Close()


class test_proxy_attribute(unittest.TestCase):
    
    """ This class tests the different types of smart returning used
    by the proxy classes.  The aim is to make the proxy behave very
    much like the un-proxied classes so checking the return types is
    important especially because lists, tuples and dictionaries have
    to be properly proxied."""
    
    def check_int(self):
        "Checking proxying of function returning int"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        self.assertEqual(a.get_int(), 1)

    def check_float(self):
        "Checking proxying of function returning float"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        self.assertEqual(a.get_float(), math.pi)

    def check_complex(self):
        "Checking proxying of function returning complex"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        self.assertEqual(a.get_complex(), 1.0 + 5j)
            
    def check_string(self):
        "Checking proxying of function returning string"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        self.assertEqual(a.get_string(), "This is a string")

    def check_list(self):
        "Checking proxying of function returning list"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        no_proxy = TestProxyAttr()
        l = a.get_list()
        l1 = no_proxy.get_list()
        self.assertEqual(l1, l)

    def check_tuple(self):
        "Checking proxying of function returning tuple"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        no_proxy = TestProxyAttr()
        t = a.get_tuple()
        t1 = no_proxy.get_tuple()
        for i in range(len(t1)):
            if type(t1[i]) is types.TupleType:
                self.assertEqual(list(t1[i]), list(t[i]))
            elif isinstance(t1[i], TestClass):
                self.assertEqual(t[i].test(), t1[i].test())
            else:
                self.assertEqual(t[i], t1[i])
    
    def check_dict(self):
        "Checking proxying of function returning dictionary"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        no_proxy = TestProxyAttr()
        d = a.get_dict()
        d1 = no_proxy.get_dict()
        self.assertEqual(d.keys(), d1.keys())
        for i in d1.keys():
            if i == 'tuple':
                self.assertEqual(list(d[i]), list(d1[i]))
            elif i == 'obj':
                self.assertEqual(d[i].test(), d1[i].test())
            else:
                self.assertEqual(d[i], d1[i])

    def check_callable(self):
        "Checking proxying of function returning callable"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        no_proxy = TestProxyAttr()
        self.assertEqual(a.get_callable()(), no_proxy.get_callable()())

    def check_attr_list(self):
        "Checking if changing list attributes work"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        a.lst.append(3)
        self.assertEqual(a.lst[-1], 3)
        
    def check_attr_dict(self):
        "Checking if changing dictionary attributes work"
        f = gui_thread.register(TestProxyAttr)
        a = f()
        a.dict['c'] = 'c'
        self.assertEqual(a.dict['c'], 'c')

############################################################

# Utility clases for running the tests and for beautification.

############################################################

class TesterApp (wxApp):
    def OnInit (self):
        f = TesterFrame(None)
        return true
    
class TesterFrame(wxFrame):
    def __init__(self, parent):
        wxFrame.__init__(self, parent, -1, "Tester")

        self.CreateStatusBar()
        sizer = wxBoxSizer(wxHORIZONTAL)
        ID = NewId()
        btn = wxButton(self, ID, "Start Test")
        EVT_BUTTON(self, ID, self.OnStart)
        msg = "Click to start running tests. "\
              "Tester Output will be shown on the shell."
        btn.SetToolTip(wxToolTip(msg))
        sizer.Add(btn, 1, wxEXPAND)
        ID = NewId()
        btn = wxButton(self, ID, "Close")
        EVT_BUTTON(self, ID, self.OnClose)
        btn.SetToolTip(wxToolTip("Click to close the tester."))
        sizer.Add(btn, 1, wxEXPAND)

        sizer.Fit(self)
        self.SetAutoLayout(true)
        self.SetSizer(sizer)
        
        self.Show(1)        

    def OnStart(self, evt):
        self.SetStatusText("Running Tests")
        test()
        self.SetStatusText("Finished Running Tests")

    def OnClose(self, evt):
        self.Close(1)

if __name__ == "__main__":
    if not threaded():        
        app = TesterApp()
        app.MainLoop()
    else:
        test()



#
# Title: Import wxPython to its own thread.
#
# Author: Pearu Peterson <pearu@cens.ioc.ee>
# Created: October, 2003
#

__all__ = ['wxPython_thread']

import os
import sys
import new
import types
import thread
import inspect
import atexit
import imp
import glob
from scipy_base import ParallelExec

def get_extmodules(pexec):
    # .. that need wrappers

    state0 = sys.modules.keys()
    _do_import(pexec)
    state1 = sys.modules.keys()

    return [k for k in state1 if k not in state0 \
            and (not hasattr(sys.modules[k],'__file__') \
                 or os.path.splitext(sys.modules[k].__file__)[1][:3]!='.py')]

def wrap_extmodule(module, call_holder):
    """ Return wrapper module.
    Current design assumes wx extension modules.
    """
    new_module = new.module(module.__name__)
    for m in inspect.getmembers(module):
        n,v = m
        t = type(v)
        if t in [types.NoneType,types.StringType,types.IntType]:
            # No wrappers are needed.
            setattr(new_module,n,v)
        elif t is types.BuiltinFunctionType:
            setattr(new_module,n,wrap_builtinfunction(v,call_holder))
        elif t is types.ClassType:
            if str(v)=='wxPython.wxc.wxPyAssertionError':
                # Do we need a wrapper for it?
                setattr(new_module,n,v)
            else:
                raise NotImplementedError,`t`
        elif n in ('wxPyCoreAPI', '_wxPyCoreAPI'):
            # wxPyCoreAPI is not used by the python part of wx, so
            # no wrapper is necessary
            setattr(new_module,n,v)
        elif n=='cvar':
            # In wx all cvar attributes are of string type:
            #print [type(getattr(v,a)) for a in get_cvar_attrs(v)]
            # So no wrappers are necessary.
            setattr(new_module,n,v)
        else:
            raise NotImplementedError,`t`
    # This gives the way how to recognize if module is a wrapper:
    #new_module._old_module = module 
    return new_module

def wrap_builtinfunction(func, call_holder):
    func_name = func.__name__
    if func_name in ('wxApp_CleanUp', 'App_CleanUp'):
        return func
    main_thread_id = call_holder.main_thread_id
    func_tmpl = """\
def %(func_name)s(*args,**kws):
    if get_ident()==%(main_thread_id)s:
        call_holder.call('call_holder.result=old_func(*args,**kws)')
        result = call_holder.result
        call_holder.result = None
        return result
    return old_func(*args,**kws)
func_code=%(func_name)s.func_code"""
    exec func_tmpl % (locals())
    globs = {'old_func':func,
             'get_ident':thread.get_ident,
             'call_holder':call_holder}
    return new.function(func_code,globs,func_name)


def _do_import(pexec):
    """This imports the wxPython modules.  This code handles both
    wxPython 2.4.x and 2.5.x versions.

    """
    sfx = [x[0] for x in imp.get_suffixes() if x[-1] is imp.C_EXTENSION]
    suffix = '.so'

    # Handle the imports for wxPython-2.5.x
    flag = 0
    for path in sys.path:
        wx_pth = os.path.join(path, 'wx')
        if os.path.exists(os.path.join(wx_pth, '__init__.py')):
            for x in sfx:
                if os.path.exists(os.path.join(wx_pth, '_core_' + x)):
                    suffix = x
                    flag = 1
                    assert not sys.modules.has_key('wx'), \
                           'wx is already imported, cannot proceed'
                    # Now do necessary stuff for wx.
                    _import_wx_core(pexec, wx_pth, suffix)
                    break

    # wxPython-2.4.x
    if not flag: 
        assert not sys.modules.has_key('wxPython'),\
               'wxPython is already imported, cannot proceed'
        pexec('import wxPython', wait=1)
        path = os.path.dirname(sys.modules['wxPython'].__file__)
        suffix = os.path.splitext(sys.modules['wxPython.wxc'].__file__)[1]
        exts = glob.glob(os.path.join(path, '*' + suffix))
        for x in [os.path.splitext(os.path.basename(i))[0] for i in exts]:
            pexec('import wxPython.%s'%x, wait=1)
        

def _import_wx_core(pexec, wx_pth, suffix):
    """Imports the core modules for wx.  This is necessary for
    wxPython-2.5.x.

    """
    # Create a dummy module in sys.modules to inhibit importing the
    # actual one.  We will reload(wx) to get it right later.
    m = new.module('wx')
    m.__file__ = os.path.join('wx_pth', '__init__.py')
    sys.modules['wx'] = m

    # Now import all the extension modules manually.
    pexec('import imp')
    libs = glob.glob(os.path.join(wx_pth, '*' + suffix))
    done = []
    for i in ['_core_', '_controls_', '_misc_', '_windows_', '_gdi_']:
        p = os.path.join(wx_pth, i + suffix)
        pexec('imp.load_dynamic("wx." + "%s", "%s")'%(i, p), wait=1)
        done.append(p)
    libs = [x for x in libs if x not in done]
    for i in libs:
        p = os.path.splitext(os.path.basename(i))[0]
        pexec('imp.load_dynamic("wx." + "%s", "%s")'%(p, i), wait=1)

    
def wxPython_thread():
    class AttrHolder: pass

    call_holder = AttrHolder()
    call_holder.call = None
    call_holder.main_thread_id = thread.get_ident()
    pexec = ParallelExec()

    # Create wrappers to wxPython extension modules:
    for name in get_extmodules(pexec):
        module = sys.modules[name]
        if module is None: # happens with gui_thread.wxPython
            continue
        sys.modules[name] = wrap_extmodule(module,call_holder)
    
    if sys.modules.has_key('wxPython.wxc'):
        pexec('import wxPython; reload(wxPython)', wait=1)
        import wxPython
    else:
        pexec('import wx; reload(wx)', wait=1)
        import wx

    pexec('from wxBackgroundApp import wxBackgroundApp')
    pexec('call_holder.call=wxBackgroundApp()',wait=1)
    # Start wxPython application in background:
    pexec('call_holder.call.run()')

    atexit.register(pexec.shutdown)
    return



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# The following is so I know about arrays:
from Numeric import *
from scipy_base.fastumath import *
from shapetest import *
from graftypes import *

class Surface :
   """s= Surface ( <keyword arguments> ) will create a surface with
   coordinates and various other characteristics. The keywords are
   as follows:
        z = <value> (required). z is a two dimensional array. If x
            and y are not specified, then z will be graphed on equally
            spaced mesh points.
        x = <value>, y = <value> (if one is present, then so must
            the other be.) If c (below) is not present, this represents
            a 3d plot. Either x and y have dimensions matching z,
            or else they are one-dimensional and x's length matches
            the first dimension of z, and y's length matches the second.
        c = <value> If present, this represents a four-dimensional
            graph which is colored according to the values of c.
            c must have the same dimensions as z.
        color_card = <value> specifies which color card you wish
            to use, e. g., "rainbowhls" (the default), "random",
            etc. Although a characteristic of a Graph2d, it can
            be a Surface characteristic (of a Narcisse plot only)
            since 'link'ed surfaces can have different color cards.
        opt_3d = <value> where <value> is a string or a sequence
            of strings giving the 3d or 4d surface characteristics.
            A surface is colored by height in z if a 3d option is
            specified, and by the value of the function c if a 4d
            option is specified. With a wire grid option, the grid
            is colored; with a flat option, the quadrilaterals set
            off by grid points are colored; with a smooth option,
            the surface itself is colored by height; and with an iso
            option, the contour lines are colored. flat and iso options
            may be used together in any combination. wire grid options
            are independent of the other options. Legal arguments for
            set_3d_options are:
            'wm'--monochrome wire grid; 'w3' and 'w4'--3d and 4d
            coloring of wire grid.
            'f3' and 'f4'--flat 3d and 4d coloring options.
            'i3' and 'i4'--3d and 4d isoline (contour line) options.
            's3' and 's4'--3d and 4d smooth coloring options.
            For Gist, a subset of the above is available:
            'wm', 's3', 'f3', and 'f4'.
         ecolor = <string> In Gist, you are allowed to specify a
            color for the edges of the mesh. (default: "fg")
         shade = (0|1) Applicable only with Gist. If 1, show the
            surface shaded according to the specified or default
            light source.
         mesh_type = <string> in one of the wire modes, tells what
            form the wire grid takes: "x"--x lines only; "y": y lines only;
            "xy": both a lines and y lines.
            Not available for Gist.
         mask = <string> controls hidden line removal. Allowed values
            are "none" : transparent graph; "min": simple
            masking; "max" : better masking; "sort": slowest but
            most sophisticated.
            Gist only does "none" and "sort".
         z_c_switch = 0 or 1 : set to 1 means switch z and c in the plot.
         z_contours_scale, c_contours_scale = "lin", "log", or "normal"
         z_contours_array, c_contours_array = actual array of numbers
            to use for contours
         number_of_z_contours, number_of_c_contours = <integer>
            specifies how many contours to use; they will be computed
            automatically based on the data.
   """

   def type (self) :
      return SurfaceType

   _SurfaceSpecError = "SurfaceSpecError"

   _color_card_dict = { "absolute" : 0 , "binary" : 1 ,
      "bluegreen" : 2 , "default" : 3 , "negative" : 4 , "positive" : 5 ,
      "rainbow" : 6 , "rainbowhls" : 7 , "random" : 8 , "redblue" : 9 ,
      "redgreen" : 10 , "shifted" : 11 }

   def __init__ ( self, * kwds , ** keywords ) :
       if len (kwds) == 1 :
          keywords = kwds[0]
       self.cell_descr = None # (this will have values for a nonstruct. mesh)
       if ( keywords.has_key ( "z" ) ) :
          self.z = keywords ["z"]
       else :
          raise self._SurfaceSpecError , \
             "Please specify z coordinates for the surface."
       if keywords.has_key ( "x" ) :
          self.x = keywords ["x"]
          if not keywords.has_key ("y") :
             raise self._SurfaceSpecError , \
                "Both x and y coordinates must be supplied."
          self.y = keywords ["y"]
       elif keywords.has_key ( "y" ) :
          raise self._SurfaceSpecError , \
             "Both x and y coordinates must be supplied."
       else :
          self.x = None
          self.y = None
       if keywords.has_key ( "c" ) :
          self.c = keywords ["c"]
          if self.x is None : # got to dummy up x and y
             self.x = arange (self.z.shape [0], typecode = Float)
             self.y = arange (self.z.shape [1], typecode = Float)
       else :
          self.c = None
       self.generic_init ( keywords )

   def generic_init ( self , keywords ) :
       if keywords.has_key ( "color_card" ) :
          if type ( keywords [ "color_card" ] ) == type ( 1 ) :
             self.color_card = keywords [ "color_card" ]
          elif self._color_card_dict.has_key ( keywords [ "color_card" ] ) :
             self.color_card = self._color_card_dict [keywords [ "color_card" ]]
          else :
             self.color_card = 7
       else :
          self.color_card = 7
       # N. B. on the above: the color card option does not apply in Gist.
       if keywords.has_key ( "mask" ) :
          self.mask = keywords ["mask"]
       else :
          self.mask = "none"
       if keywords.has_key ( "opt_3d" ) :
          self.opt_3d = keywords ["opt_3d"]
       else :
          self.opt_3d = "wm"
       if keywords.has_key ( "mesh_type" ) :
          self.mesh_type = keywords ["mesh_type"]
       else :
          self.mesh_type = "xy"
       if keywords.has_key ( "z_c_switch" ) :
          self.z_c_switch = keywords ["z_c_switch"]
       else :
          self.z_c_switch = 0
       if keywords.has_key ( "z_contours_scale" ) :
          self.z_contours_scale = keywords ["z_contours_scale"]
       else :
          self.z_contours_scale = "lin"
       if keywords.has_key ( "c_contours_scale" ) :
          self.c_contours_scale = keywords ["c_contours_scale"]
       else :
          self.c_contours_scale = "lin"
       if keywords.has_key ( "number_of_z_contours") :
          self.number_of_z_contours = keywords ["number_of_z_contours"]
       else :
          self.number_of_z_contours = None
       if keywords.has_key ( "number_of_c_contours" ) :
          self.number_of_c_contours = keywords ["number_of_c_contours"]
       else :
          self.number_of_c_contours = None
       if keywords.has_key ( "z_contours_array" ) :
          self.z_contours_array = keywords ["z_contours_array"]
       else :
          self.z_contours_array = None
       if keywords.has_key ( "c_contours_array" ) :
          self.c_contours_array = keywords ["c_contours_array"]
       else :
          self.c_contours_array = None
       if keywords.has_key ( "ecolor" ) :
          self.ecolor = keywords ["ecolor"]
       else :
          self.ecolor = "fg"
       if keywords.has_key ( "shade" ) :
          self.shade = keywords ["shade"]
       else :
          self.shade = 0

   def new ( self, ** keywords ) :
       """ new (...keyword arguments...) allows you to reuse a
       previously existing surface.
       """
       del self.x, self.y, self.z, self.c, self.color_card, self.opt_3d, \
           self.mask, self.z_c_switch, self.z_contours_scale, \
           self.c_contours_scale, self.z_contours_array, \
           self.c_contours_array, self.number_of_z_contours, \
           self.number_of_c_contours, self.mesh_type
       self.__init__ ( keywords )

   def set ( self , ** keywords ) :
       """ set (...keyword arguments...) allows you to set individual
       surface characteristics. No error checking is done.
       """
       for k in keywords.keys ():
          if k == "x" :
             self.x = keywords ["x"]
          elif k == "y" :
             self.y = keywords ["y"]
          elif k == "z" :
             self.z = keywords ["z"]
          elif k == "c" :
             self.c = keywords ["c"]
          else :
             setattr (self, k, keywords [k])



from Numeric import *
from scipy_base.scipy_base.fastumath import *
import vq_c as vq
  
def python_vq(all_data,code_book):    
    import time
    t1 = time.time()
    codes1,dist1 = vq.vq(all_data,code_book)    
    t2 = time.time()
    print 'fast (double):', t2 - t1
    print '  first codes:', codes1[:5]
    print '  first dist:', dist1[:5]
    print '  last codes:', codes1[-5:]
    print '  last dist:', dist1[-5:]
    float_obs = all_data.astype(Float32)
    float_code = code_book.astype(Float32)
    t1 = time.time()
    codes1,dist1 = vq.vq(float_obs,float_code)    
    t2 = time.time()
    print 'fast (float):', t2 - t1
    print '  first codes:', codes1[:5]
    print '  first dist:', dist1[:5]
    print '  last codes:', codes1[-5:]
    print '  last dist:', dist1[-5:]

    return codes1,dist1

def read_data(name):    
    f = open(name,'r')
    data = []
    for line in f.readlines():
        data.append(map(float,string.split(line)))
    f.close()
    return array(data)        

def main():
    import scipy.stats
    scipy.stats.seed(1000,1000)
    Ncodes = 40
    Nfeatures = 16
    Nobs = 4000        
    code_book = RandomArray.normal(0,1,(Ncodes,Nfeatures))
    features = RandomArray.normal(0,1,(Nobs,Nfeatures))
    codes,dist = python_vq(features,code_book)
   
if __name__ == '__main__':
    main()    



# This code allows one to use SWIG wrapped objects from weave.  This
# code is specific to SWIG-1.3 and above where things are different.
# The code is basically all copied out from the SWIG wrapper code but
# it has been hand edited for brevity.
#
# Prabhu Ramachandran <prabhu@aero.iitm.ernet.in>

swigptr2_code = """

#include "Python.h"

/*************************************************************** -*- c -*-
 * python/precommon.swg
 *
 * Rename all exported symbols from common.swg, to avoid symbol
 * clashes if multiple interpreters are included
 *
 ************************************************************************/

#define SWIG_TypeCheck       SWIG_Python_TypeCheck
#define SWIG_TypeCast        SWIG_Python_TypeCast
#define SWIG_TypeName        SWIG_Python_TypeName
#define SWIG_TypeQuery       SWIG_Python_TypeQuery
#define SWIG_PackData        SWIG_Python_PackData 
#define SWIG_UnpackData      SWIG_Python_UnpackData 


/***********************************************************************
 * common.swg
 *
 *     This file contains generic SWIG runtime support for pointer
 *     type checking as well as a few commonly used macros to control
 *     external linkage.
 *
 * Author : David Beazley (beazley@cs.uchicago.edu)
 *
 * Copyright (c) 1999-2000, The University of Chicago
 * 
 * This file may be freely redistributed without license or fee provided
 * this copyright message remains intact.
 ************************************************************************/

#include <string.h>

#if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
#  if defined(_MSC_VER) || defined(__GNUC__)
#    if defined(STATIC_LINKED)
#      define SWIGEXPORT(a) a
#      define SWIGIMPORT(a) extern a
#    else
#      define SWIGEXPORT(a) __declspec(dllexport) a
#      define SWIGIMPORT(a) extern a
#    endif
#  else
#    if defined(__BORLANDC__)
#      define SWIGEXPORT(a) a _export
#      define SWIGIMPORT(a) a _export
#    else
#      define SWIGEXPORT(a) a
#      define SWIGIMPORT(a) a
#    endif
#  endif
#else
#  define SWIGEXPORT(a) a
#  define SWIGIMPORT(a) a
#endif

#ifdef SWIG_GLOBAL
#  define SWIGRUNTIME(a) SWIGEXPORT(a)
#else
#  define SWIGRUNTIME(a) static a
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void *(*swig_converter_func)(void *);
typedef struct swig_type_info *(*swig_dycast_func)(void **);

typedef struct swig_type_info {
  const char             *name;
  swig_converter_func     converter;
  const char             *str;
  void                   *clientdata;
  swig_dycast_func        dcast;
  struct swig_type_info  *next;
  struct swig_type_info  *prev;
} swig_type_info;

#ifdef SWIG_NOINCLUDE

SWIGIMPORT(swig_type_info *) SWIG_TypeCheck(char *c, swig_type_info *);
SWIGIMPORT(void *)           SWIG_TypeCast(swig_type_info *, void *);
SWIGIMPORT(const char *)     SWIG_TypeName(const swig_type_info *);
SWIGIMPORT(swig_type_info *) SWIG_TypeQuery(const char *);
SWIGIMPORT(char *)           SWIG_PackData(char *, void *, int);
SWIGIMPORT(char *)           SWIG_UnpackData(char *, void *, int);

#else

static swig_type_info *swig_type_list = 0;

/* Check the typename */
SWIGRUNTIME(swig_type_info *) 
SWIG_TypeCheck(char *c, swig_type_info *ty) {
  swig_type_info *s;
  if (!ty) return 0;        /* Void pointer */
  s = ty->next;             /* First element always just a name */
  do {
    if (strcmp(s->name,c) == 0) {
      if (s == ty->next) return s;
      /* Move s to the top of the linked list */
      s->prev->next = s->next;
      if (s->next) {
        s->next->prev = s->prev;
      }
      /* Insert s as second element in the list */
      s->next = ty->next;
      if (ty->next) ty->next->prev = s;
      ty->next = s;
      s->prev = ty;
      return s;
    }
    s = s->next;
  } while (s && (s != ty->next));
  return 0;
}

/* Cast a pointer up an inheritance hierarchy */
SWIGRUNTIME(void *) 
SWIG_TypeCast(swig_type_info *ty, void *ptr) {
  if ((!ty) || (!ty->converter)) return ptr;
  return (*ty->converter)(ptr);
}

/* Return the name associated with this type */
SWIGRUNTIME(const char *)
SWIG_TypeName(const swig_type_info *ty) {
  return ty->name;
}

/* 
   Compare two type names skipping the space characters, therefore
   "char*" == "char *" and "Class<int>" == "Class<int >", etc.

   Return 0 when the two name types are equivalent, as in
   strncmp, but skipping ' '.
*/
static int
SWIG_TypeNameComp(const char *f1, const char *l1,
		  const char *f2, const char *l2) {
  for (;(f1 != l1) && (f2 != l2); ++f1, ++f2) {
    while ((*f1 == ' ') && (f1 != l1)) ++f1;
    while ((*f2 == ' ') && (f2 != l2)) ++f2;
    if (*f1 != *f2) return *f1 - *f2;
  }
  return (l1 - f1) - (l2 - f2);
}

/*
  Check type equivalence in a name list like <name1>|<name2>|...
*/
static int
SWIG_TypeEquiv(const char *nb, const char *tb) {
  int equiv = 0;
  const char* te = tb + strlen(tb);
  const char* ne = nb;
  while (!equiv && *ne) {
    for (nb = ne; *ne; ++ne) {
      if (*ne == '|') break;
    }
    equiv = SWIG_TypeNameComp(nb, ne, tb, te) == 0;
    if (*ne) ++ne;
  }
  return equiv;
}
  

/* Search for a swig_type_info structure */
SWIGRUNTIME(swig_type_info *)
SWIG_TypeQuery(const char *name) {
  swig_type_info *ty = swig_type_list;
  while (ty) {
    if (ty->str && (SWIG_TypeEquiv(ty->str,name))) return ty;
    if (ty->name && (strcmp(name,ty->name) == 0)) return ty;
    ty = ty->prev;
  }
  return 0;
}

/* Pack binary data into a string */
SWIGRUNTIME(char *)
SWIG_PackData(char *c, void *ptr, int sz) {
  static char hex[17] = "0123456789abcdef";
  int i;
  unsigned char *u = (unsigned char *) ptr;
  register unsigned char uu;
  for (i = 0; i < sz; i++,u++) {
    uu = *u;
    *(c++) = hex[(uu & 0xf0) >> 4];
    *(c++) = hex[uu & 0xf];
  }
  return c;
}

/* Unpack binary data from a string */
SWIGRUNTIME(char *)
SWIG_UnpackData(char *c, void *ptr, int sz) {
  register unsigned char uu = 0;
  register int d;
  unsigned char *u = (unsigned char *) ptr;
  int i;
  for (i = 0; i < sz; i++, u++) {
    d = *(c++);
    if ((d >= '0') && (d <= '9'))
      uu = ((d - '0') << 4);
    else if ((d >= 'a') && (d <= 'f'))
      uu = ((d - ('a'-10)) << 4);
    d = *(c++);
    if ((d >= '0') && (d <= '9'))
      uu |= (d - '0');
    else if ((d >= 'a') && (d <= 'f'))
      uu |= (d - ('a'-10));
    *u = uu;
  }
  return c;
}

#endif

#ifdef __cplusplus
}
#endif

/***********************************************************************
 * python.swg
 *
 *     This file contains the runtime support for Python modules
 *     and includes code for managing global variables and pointer
 *     type checking.
 *
 * Author : David Beazley (beazley@cs.uchicago.edu)
 ************************************************************************/

#include "Python.h"

#ifdef __cplusplus
extern "C" {
#endif

#define SWIG_PY_INT     1
#define SWIG_PY_FLOAT   2
#define SWIG_PY_STRING  3
#define SWIG_PY_POINTER 4
#define SWIG_PY_BINARY  5

/* Flags for pointer conversion */

#define SWIG_POINTER_EXCEPTION     0x1
#define SWIG_POINTER_DISOWN        0x2

/* Exception handling in wrappers */
#define SWIG_fail   goto fail

/* Constant information structure */
typedef struct swig_const_info {
    int type;
    char *name;
    long lvalue;
    double dvalue;
    void   *pvalue;
    swig_type_info **ptype;
} swig_const_info;

/* Common SWIG API */
#define SWIG_ConvertPtr(obj, pp, type, flags) \
  SWIG_Python_ConvertPtr(obj, pp, type, flags)
#define SWIG_NewPointerObj(p, type, flags) \
  SWIG_Python_NewPointerObj(p, type, flags)
#define SWIG_MustGetPtr(p, type, argnum, flags) \
  SWIG_Python_MustGetPtr(p, type, argnum, flags)
 

typedef double (*py_objasdbl_conv)(PyObject *obj);

#ifdef SWIG_NOINCLUDE

SWIGIMPORT(int)               SWIG_Python_ConvertPtr(PyObject *, void **, swig_type_info *, int);
SWIGIMPORT(PyObject *)        SWIG_Python_NewPointerObj(void *, swig_type_info *,int own);
SWIGIMPORT(void *)            SWIG_Python_MustGetPtr(PyObject *, swig_type_info *, int, int);

#else


/* Convert a pointer value */
SWIGRUNTIME(int)
SWIG_Python_ConvertPtr(PyObject *obj, void **ptr, swig_type_info *ty, int flags) {
  swig_type_info *tc;
  char  *c = 0;
  static PyObject *SWIG_this = 0;
  int    newref = 0;
  PyObject  *pyobj = 0;

  if (!obj) return 0;
  if (obj == Py_None) {
    *ptr = 0;
    return 0;
  }
#ifdef SWIG_COBJECT_TYPES
  if (!(PyCObject_Check(obj))) {
    if (!SWIG_this)
      SWIG_this = PyString_FromString("this");
    pyobj = obj;
    obj = PyObject_GetAttr(obj,SWIG_this);
    newref = 1;
    if (!obj) goto type_error;
    if (!PyCObject_Check(obj)) {
      Py_DECREF(obj);
      goto type_error;
    }
  }  
  *ptr = PyCObject_AsVoidPtr(obj);
  c = (char *) PyCObject_GetDesc(obj);
  if (newref) Py_DECREF(obj);
  goto cobject;
#else
  if (!(PyString_Check(obj))) {
    if (!SWIG_this)
      SWIG_this = PyString_FromString("this");
    pyobj = obj;
    obj = PyObject_GetAttr(obj,SWIG_this);
    newref = 1;
    if (!obj) goto type_error;
    if (!PyString_Check(obj)) {
      Py_DECREF(obj);
      goto type_error;
    }
  } 
  c = PyString_AsString(obj);
  /* Pointer values must start with leading underscore */
  if (*c != '_') {
    *ptr = (void *) 0;
    if (strcmp(c,"NULL") == 0) {
      if (newref) { Py_DECREF(obj); }
      return 0;
    } else {
      if (newref) { Py_DECREF(obj); }
      goto type_error;
    }
  }
  c++;
  c = SWIG_UnpackData(c,ptr,sizeof(void *));
  if (newref) { Py_DECREF(obj); }
#endif

#ifdef SWIG_COBJECT_TYPES
cobject:
#endif

  if (ty) {
    tc = SWIG_TypeCheck(c,ty);
    if (!tc) goto type_error;
    *ptr = SWIG_TypeCast(tc,(void*) *ptr);
  }

  if ((pyobj) && (flags & SWIG_POINTER_DISOWN)) {
    PyObject *zero = PyInt_FromLong(0);
    PyObject_SetAttrString(pyobj,(char*)"thisown",zero);
    Py_DECREF(zero);
  }
  return 0;

type_error:
  PyErr_Clear();
  if (flags & SWIG_POINTER_EXCEPTION) {
    if (ty && c) {
      PyErr_Format(PyExc_TypeError, 
		   "Type error. Got %s, expected %s",
		   c, ty->name);
    } else {
      PyErr_SetString(PyExc_TypeError,"Expected a pointer");
    }
  }
  return -1;
}

/* Convert a pointer value, signal an exception on a type mismatch */
SWIGRUNTIME(void *)
SWIG_Python_MustGetPtr(PyObject *obj, swig_type_info *ty, int argnum, int flags) {
  void *result;
  SWIG_Python_ConvertPtr(obj, &result, ty, flags | SWIG_POINTER_EXCEPTION);
  return result;
}

/* Create a new pointer object */
SWIGRUNTIME(PyObject *)
SWIG_Python_NewPointerObj(void *ptr, swig_type_info *type, int own) {
  PyObject *robj;
  if (!ptr) {
    Py_INCREF(Py_None);
    return Py_None;
  }
#ifdef SWIG_COBJECT_TYPES
  robj = PyCObject_FromVoidPtrAndDesc((void *) ptr, (char *) type->name, NULL);
#else
  {
    char result[1024];
    char *r = result;
    *(r++) = '_';
    r = SWIG_PackData(r,&ptr,sizeof(void *));
    strcpy(r,type->name);
    robj = PyString_FromString(result);
  }
#endif
  if (!robj || (robj == Py_None)) return robj;
  if (type->clientdata) {
    PyObject *inst;
    PyObject *args = Py_BuildValue((char*)"(O)", robj);
    Py_DECREF(robj);
    inst = PyObject_CallObject((PyObject *) type->clientdata, args);
    Py_DECREF(args);
    if (inst) {
      if (own) {
        PyObject *n = PyInt_FromLong(1);
        PyObject_SetAttrString(inst,(char*)"thisown",n);
        Py_DECREF(n);
      }
      robj = inst;
    }
  }
  return robj;
}

#endif

#ifdef __cplusplus
}
#endif

"""



"""
Special Functions
=================

  Airy Functions

    airy     -- Airy functions and their derivatives.
    airye    -- Exponentially scaled Airy functions
    ai_zeros -- **Zeros of Airy functions Ai(x) and Ai'(x)
    bi_zeros -- **Zeros of Airy functions Bi(x) and Bi'(x)

  Elliptic Functions and Integrals

    ellipj    -- Jacobian elliptic functions
    ellipk    -- Complete elliptic integral of the first kind.
    ellipkinc -- Incomplete elliptic integral of the first kind.
    ellipe    -- Complete elliptic integral of the second kind.
    ellipeinc -- Incomplete elliptic integral of the second kind.

  Bessel Functions

    jn       -- Bessel function of integer order and real argument.
    jv       -- Bessel function of real-valued order and complex argument.
    jve      -- Exponentially scaled Bessel function.
    yn       -- Bessel function of second kind (integer order).
    yv       -- Bessel function of the second kind (real-valued order).
    yve      -- Exponentially scaled Bessel function of the second kind.
    kn       -- Modified Bessel function of the third kind (integer order).
    kv       -- Modified Bessel function of the third kind (real order).
    kve      -- Exponentially scaled modified Bessel function of the
                  third kind.
    iv       -- Modified Bessel function.
    ive      -- Exponentially scaled modified Bessel function.
    hankel1  -- Hankel function of the first kind.
    hankel1e -- Exponentially scaled Hankel function of the first kind.
    hankel2  -- Hankel function of the second kind.
    hankel2e -- Exponentially scaled Hankel function of the second kind.

    lmbda       -- **Sequence of lambda functions with arbitrary order v.

  Zeros of Bessel Functions

    jnjnp_zeros -- **Zeros of integer-order Bessel functions and derivatives
                       sorted in order.
    jnyn_zeros  -- **Zeros of integer-order Bessel functions and derivatives
                       as separate arrays.
    jn_zeros    -- **Zeros of Jn(x)
    jnp_zeros   -- **Zeros of Jn'(x)
    yn_zeros    -- **Zeros of Yn(x)
    ynp_zeros   -- **Zeros of Yn'(x)
    y0_zeros    -- **Complex zeros: Y0(z0)=0 and values of Y0'(z0)
    y1_zeros    -- **Complex zeros: Y1(z1)=0 and values of Y1'(z1)
    y1p_zeros   -- **Complex zeros of Y1'(z1')=0 and values of Y1(z1')

  Faster versions of common Bessel Functions.
    
    j0       -- Bessel function of order 0.
    j1       -- Bessel function of order 1.
    y0       -- Bessel function of second kind of order 0.
    y1       -- Bessel function of second kind of order 1.
    i0       -- Modified Bessel function of order 0.
    i0e      -- Exponentially scaled modified Bessel function of order 0.
    i1       -- Modified Bessel function of order 1.
    i1e      -- Exponentially scaled modified Bessel function of order 1.
    k0       -- Modified Bessel function of the third kind of order 0.
    k0e      -- Exponentially scaled modified Bessel function of the
                  third kind of order 0.
    k1       -- Modified Bessel function of the third kind of order 1.
    k1e      -- Exponentially scaled modified Bessel function of the
                  third kind of order 1.

  Integrals of Bessel Functions.

    itj0y0     -- Basic integrals of j0 and y0 from 0 to x.
    it2j0y0    -- Integrals of (1-j0(t))/t from 0 to x and
                    y0(t)/t from x to inf.
    iti0k0     -- Basic integrals of i0 and k0 from 0 to x.
    it2i0k0    -- Integrals of (i0(t)-1)/t from 0 to x and
                    k0(t)/t from x to inf.
    besselpoly -- Integral of a bessel function: Jv(2*a*x) * x**lambda
                    from x=0 to 1.

  Derivatives of Bessel Functions.

    jvp     -- Nth derivative of Jv(v,z)
    yvp     -- Nth derivative of Yv(v,z)
    kvp     -- Nth derivative of Kv(v,z)
    ivp     -- Nth derivative of Iv(v,z)
    h1vp    -- Nth derivative of H1v(v,z)
    h2vp    -- Nth derivative of H2v(v,z)    

  Spherical Bessel Functions

    sph_jn   -- **Sequence of spherical Bessel functions, jn(z)
    sph_yn   -- **Sequence of spherical Bessel functions, yn(z)
    sph_jnyn -- **Sequence of spherical Bessel functions, jn(z) and yn(z)
    sph_in   -- **Sequence of spherical Bessel functions, in(z)
    sph_kn   -- **Sequence of spherical Bessel functions, kn(z)
    sph_inkn -- **Sequence of spherical Bessel functions, in(z) and kn(z)

  Ricatti-Bessel Functions

    ricatti_jn -- **Sequence of Ricatti-Bessel functions of first kind.
    ricatti_yn -- **Sequence of Ricatti-Bessel functions of second kind.

  Struve Functions

    struve       -- Struve function --- Hv(x)
    modstruve    -- Modified struve function --- Lv(x)
    itstruve0    -- Integral of H0(t) from 0 to x
    it2struve0   -- Integral of H0(t)/t from x to Inf.
    itmodstruve0 -- Integral of L0(t) from 0 to x.
    
        
  Raw Statistical Functions (Friendly versions in scipy.stats)

    bdtr       -- Sum of terms 0 through k of of the binomial pdf.
    bdtrc      -- Sum of terms k+1 through n of the binomial pdf.
    bdtri      -- Inverse of bdtr
    btdtr      -- Integral from 0 to x of beta pdf.
    btdtri     -- Quantiles of beta distribution
    fdtr       -- Integral from 0 to x of F pdf.
    fdtrc      -- Integral from x to infinity under F pdf.
    fdtri      -- Inverse of fdtrc
    gdtr       -- Integral from 0 to x of gamma pdf.
    gdtrc      -- Integral from x to infinity under gamma pdf.
    gdtri      -- Quantiles of gamma distribution
    nbdtr      -- Sum of terms 0 through k of the negative binomial pdf.
    nbdtrc     -- Sum of terms k+1 to infinity under negative binomial pdf.
    nbdtri     -- Inverse of nbdtr
    pdtr       -- Sum of terms 0 through k of the Poisson pdf.
    pdtrc      -- Sum of terms k+1 to infinity of the Poisson pdf.
    pdtri      -- Inverse of pdtr
    stdtr      -- Integral from -infinity to t of the Student-t pdf.
    stdtri     -- Inverse of stdtr (quantiles)
    chdtr      -- Integral from 0 to x of the Chi-square pdf.
    chdtrc     -- Integral from x to infnity of Chi-square pdf.
    chdtri     -- Inverse of chdtrc.
    ndtr       -- Integral from -infinity to x of standard normal pdf
    ndtri      -- Inverse of ndtr (quantiles)
    smirnov    -- Kolmogorov-Smirnov complementary CDF for one-sided
                    test statistic (Dn+ or Dn-)
    smirnovi   -- Inverse of smirnov.
    kolmogorov -- The complementary CDF of the (scaled) two-sided test
                          statistic (Kn*) valid for large n.
    kolmogi    -- Inverse of kolmogorov
    tklmbda    -- Tukey-Lambda CDF
      
  Gamma and Related Functions

    gamma        -- Gamma function.
    gammaln      -- Log of the absolute value of the gamma function.
    gammainc     -- Incomplete gamma integral.
    gammaincinv  -- Inverse of gammainc.
    gammaincc    -- Complemented incomplete gamma integral.
    gammainccinv -- Inverse of gammaincc.
    beta         -- Beta function.
    betaln       -- Log of the absolute value of the beta function.
    betainc      -- Incomplete beta integral.
    betaincinv   -- Inverse of betainc.
    betaincinva  -- Inverse (in first argument, a) of betainc
    betaincinvb  -- Inverse (in first argument, b) of betainc        
    psi(digamma) -- Logarithmic derivative of the gamma function.
    rgamma       -- One divided by the gamma function.
    polygamma    -- Nth derivative of psi function.

  Error Function and Fresnel Integrals
  
    erf           -- Error function.
    erfc          -- Complemented error function (1- erf(x))
    erfinv        -- Inverse of error function
    erfcinv       -- Inverse of erfc
    erf_zeros     -- **Complex zeros of erf(z)
    fresnel       -- Fresnel sine and cosine integrals.
    fresnel_zeros -- Complex zeros of both Fresnel integrals
    fresnelc_zeros -- **Complex zeros of fresnel cosine integrals
    fresnels_zeros -- **Complex zeros of fresnel sine integrals
    modfresnelp   -- Modified Fresnel integrals F_+(x) and K_+(x)
    modfresnelm   -- Modified Fresnel integrals F_-(x) and K_-(x)

  Legendre Functions

    lpn      -- **Legendre Functions (polynomials) of the first kind
    lqn      -- **Legendre Functions of the second kind.
    lpmn     -- **Associated Legendre Function of the first kind.
    lqmn     -- **Associated Legendre Function of the second kind.
    lpmv     -- Associated Legendre Function of arbitrary non-negative
                   degree v.
    sph_harm -- Spherical Harmonics (complex-valued) Y^m_n(theta,phi)

  Orthogonal polynomials  --- 15 types
   ** These functions all return a polynomial class which can then be
      evaluated:  vals = chebyt(n)(x)
      This class also has an attribute 'weights' which
      return the roots, weights, and total weights for the appropriate 
      form of Gaussian quadrature.  These are returned in an n x 3 array with roots
      in the first column, weights in the second column, and total weights in the final
      column

    legendre    -- **Legendre polynomial P_n(x) (lpn -- for function).
    chebyt      -- **Chebyshev polynomial T_n(x)
    chebyu      -- **Chebyshev polynomial U_n(x)
    chebyc      -- **Chebyshev polynomial C_n(x)
    chebys      -- **Chebyshev polynomial S_n(x)
    jacobi      -- **Jacobi polynomial P^(alpha,beta)_n(x)
    laguerre    -- **Laguerre polynomial, L_n(x)
    genlaguerre -- **Generalized (Associated) Laguerre polynomial, L^alpha_n(x)
    hermite     -- **Hermite polynomial H_n(x)
    hermitenorm -- **Normalized Hermite polynomial, He_n(x)
    gegenbauer  -- **Gegenbauer (Ultraspherical) polynomials, C^(alpha)_n(x)
    sh_legendre -- **shifted Legendre polynomial, P*_n(x)
    sh_chebyt   -- **shifted Chebyshev polynomial, T*_n(x)
    sh_chebyu   -- **shifted Chebyshev polynomial, U*_n(x)
    sh_jacobi   -- **shifted Jacobi polynomial, J*_n(x) = G^(p,q)_n(x)
      
  HyperGeometric Functions

    hyp2f1   -- Gauss hypergeometric function (2F1)
    hyp1f1   -- Confluent hypergeometric function (1F1)
    hyperu   -- Confluent hypergeometric function (U)
    hyp0f1   -- Confluent hypergeometric limit function (0F1)
    hyp2f0   -- Hypergeometric function (2F0) 
    hyp1f2   -- Hypergeometric function (1F2)
    hyp3f0   -- Hypergeometric function (3F0)

  Parabolic Cylinder Functions

    pbdv     -- Parabolic cylinder function Dv(x) and derivative.
    pbvv     -- Parabolic cylinder function Vv(x) and derivative. 
    pbwa     -- Parabolic cylinder function W(a,x) and derivative.
    pbdv_seq -- **Sequence of parabolic cylinder functions Dv(x)
    pbvv_seq -- **Sequence of parabolic cylinder functions Vv(x)
    pbdn_seq -- **Sequence of parabolic cylinder functions Dn(z), complex z

  mathieu and Related Functions (and derivatives)

    mathieu_a       -- Characteristic values for even solution (ce_m)
    mathieu_b       -- Characteristic values for odd solution (se_m)
    mathieu_even_coef -- **sequence of expansion coefficients for even solution
    mathieu_odd_coef  -- **sequence of expansion coefficients for odd solution
       ** All the following return both function and first derivative **
    mathieu_cem     -- Even mathieu function
    mathieu_sem     -- Odd mathieu function
    mathieu_modcem1 -- Even modified mathieu function of the first kind
    mathieu_modcem2 -- Even modified mathieu function of the second kind
    mathieu_modsem1 -- Odd modified mathieu function of the first kind
    mathieu_modsem2 -- Odd modified mathieu function of the second kind

  Spheroidal Wave Functions

    pro_ang1   -- Prolate spheroidal angular function of the first kind
    pro_rad1   -- Prolate spheroidal radial function of the first kind
    pro_rad2   -- Prolate spheroidal radial function of the second kind
    obl_ang1   -- Oblate spheroidal angluar function of the first kind
    obl_rad1   -- Oblate spheroidal radial function of the first kind
    obl_rad2   -- Oblate spheroidal radial function of the second kind
    pro_cv     -- Compute characteristic value for prolate functions
    obl_cv     -- Compute characteristic value for oblate functions
    pro_cv_seq -- Compute sequence of prolate characteristic values
    obl_cv_seq -- Compute sequence of oblate characteristic values    
     ** The following functions require pre-computed characteristic values **
    pro_ang1_cv -- Prolate spheroidal angular function of the first kind
    pro_rad1_cv -- Prolate spheroidal radial function of the first kind
    pro_rad2_cv -- Prolate spheroidal radial function of the second kind
    obl_ang1_cv -- Oblate spheroidal angluar function of the first kind
    obl_rad1_cv -- Oblate spheroidal radial function of the first kind
    obl_rad2_cv -- Oblate spheroidal radial function of the second kind

  Kelvin Functions
  
    kelvin       -- All Kelvin functions (order 0) and derivatives.
    kelvin_zeros -- **Zeros of All Kelvin functions (order 0) and derivatives
    ber          -- Kelvin function ber x
    bei          -- Kelvin function bei x
    berp         -- Derivative of Kelvin function ber x
    beip         -- Derivative of Kelvin function bei x
    ker          -- Kelvin function ker x
    kei          -- Kelvin function kei x
    kerp         -- Derivative of Kelvin function ker x
    keip         -- Derivative of Kelvin function kei x
    ber_zeros    -- **Zeros of Kelvin function bei x
    bei_zeros    -- **Zeros of Kelvin function ber x
    berp_zeros   -- **Zeros of derivative of Kelvin function ber x
    beip_zeros   -- **Zeros of derivative of Kelvin function bei x
    ker_zeros    -- **Zeros of Kelvin function kei x
    kei_zeros    -- **Zeros of Kelvin function ker x
    kerp_zeros   -- **Zeros of derivative of Kelvin function ker x
    keip_zeros   -- **Zeros of derivative of Kelvin function kei x
  
  Other Special Functions

    expn         -- Exponential integral.
    exp1         -- Exponential integral of order 1 (for complex argument)
    expi         -- Another exponential integral -- Ei(x)
    wofz         -- Fadeeva function.
    dawsn        -- Dawson's integral.
    shichi       -- Hyperbolic sine and cosine integrals.
    sici         -- Integral of the sinc and "cosinc" functions.
    spence       -- Dilogarithm integral.
    zeta         -- Riemann zeta function of two arguments.
    zetac        -- 1.0 - standard Riemann zeta function.

  Convenience Functions

    cbrt     -- Cube root.
    exp10    -- 10 raised to the x power.
    exp2     -- 2 raised to the x power.
    radian   -- radian angle given degrees, minutes, and seconds.
    cosdg    -- cosine of the angle given in degrees.
    sindg    -- sine of the angle given in degrees.
    tandg    -- tangent of the angle given in degrees.
    cotdg    -- cotangent of the angle given in degrees.
    log1p    -- log(1+x)
    expm1    -- exp(x)-1
    cosm1    -- cos(x)-1
    round    -- round the argument to the nearest integer. If argument
                 ends in 0.5 exactly, pick the nearest even integer.

  ** in the description indicates a function which is not a universal
  function and does not follow broadcasting and automatic
  array-looping rules.

   Error handling:

      Errors are handled by returning nans, or other appropriate values.
      Some of the special function routines will print an error message
      when an error occurs.  By default this printing
      is disabled.  To enable such messages use errprint(1)
      To disable such messages use errprint(0).

      Example:
      >>> print scipy.special.bdtr(-1,10,0.3)
      >>> scipy.special.errprint(1)
      >>> print scipy.special.bdtr(-1,10,0.3)
 
"""

postpone_import = 1
global_symbols = ['isinf','isfinite','isnan']



"""
Signal Processing Tools
=======================

 Convolution:
 
    convolve      --  N-dimensional convolution.
    correlate     --  N-dimensional correlation.
    convolve2d    --  2-dimensional convolution (more options).
    correlate2d   --  2-dimensional correlation (more options).
    sepfir2d      --  Convolve with a 2-D separable FIR filter.

 B-splines:
    
    bspline       --  B-spline basis function of order n.
    gauss_spline  --  Gaussian approximation to the B-spline basis function.
    cspline1d     --  Coefficients for 1-D cubic (3rd order) B-spline.
    qspline1d     --  Coefficients for 1-D quadratic (2nd order) B-spline.
    cspline2d     --  Coefficients for 2-D cubic (3rd order) B-spline.
    qspline2d     --  Coefficients for 2-D quadratic (2nd order) B-spline.
    spline_filter --  Smoothing spline (cubic) filtering of a rank-2 array. 

 Filtering:

    order_filter  --  N-dimensional order filter.
    medfilt       --  N-dimensional median filter.
    medfilt2      --  2-dimensional median filter (faster).
    wiener        --  N-dimensional wiener filter.

    symiirorder1  --  2nd-order IIR filter (cascade of first-order systems).
    symiirorder2  --  4th-order IIR filter (cascade of second-order systems).
    lfilter       --  1-dimensional FIR and IIR digital linear filtering.

    hilbert       --- Compute the analytic signal of a 1-d signal.
    get_window    --- Create FIR window.

    detrend       --- Remove linear and/or constant trends from data.

 Filter design:
 
    remez         --  Optimal FIR filter design.
    firwin        --- Windowed FIR filter design.    
    iirdesign     --- IIR filter design given bands and gains
    iirfilter     --- IIR filter design given order and critical frequencies
    freqs         --- Analog filter frequency response
    freqz         --- Digital filter frequency response


 matlab-style IIR filter design:
 
    butter (buttord)  -- Butterworth
    cheby1 (cheb1ord) -- Chebyshev Type I
    cheby2 (cheb2ord) -- Chebyshev Type II
    ellip (ellipord)  -- Elliptic (Cauer)
    bessel            -- Bessel (no order selection available -- try butterod)

 Linear Systems:

    lti -- linear time invariant system object.
    lsim -- continuous-time simulation of output to linear system.
    impulse -- impulse response of linear, time-invariant (LTI) system.
    step -- step response of continous-time LTI system.

 LTI Reresentations:
 
    tf2zpk -- transfer function to zero-pole-gain.
    zpk2tf -- zero-pole-gain to transfer function.
    tf2ss -- transfer function to state-space.
    ss2tf -- state-pace to transfer function.
    zpk2ss -- zero-pole-gain to state-space.
    ss2zpk -- state-space to pole-zero-gain.    
"""

postpone_import = 1



#!/usr/bin/env python
#  $Id: tktest.py,v 1.1 2003/09/30 17:32:40 travo Exp $
#  -----------------------------------------------------------------
#
#  NAME:     tktest.py
#
#  PURPOSE:  Test interaction when combining Tkinter and PyGist
#
#  NOTES:
#  Tkinter and PyGist do not work together. 
#  Problem:  Tkinter does not have a proper idle event.
#  Responds to Tk events and Gist mouse events, but:
#  - Gist window does not update properly 
#  - Python command line input is no longer accepted
#
#  -----------------------------------------------------------------

import Tkinter
import sys
import tkgist
from gist import *
import gistdemolow

def DrawOval(Event):
    # Event.widget will be the main canvas:
    Event.widget.create_oval(Event.x-5,Event.y-5, Event.x+5,Event.y+5)
def DrawRectangle(Event):
    Event.widget.create_rectangle(Event.x-5,Event.y-5, Event.x+5,Event.y+5)
def MoveButton(Side):
    # The methods pack_forget() and grid_forget() unpack
    # a widget, but (unlike the destroy() method)
    # do not destroy it; it can be re-displayed later.
    QuitButton.pack_forget()
    QuitButton.pack(side=Side)

def plot():
   print ".. Gist button pressed"
   plg([0,1])

def main():
   root=Tkinter.Tk()
   MainCanvas=Tkinter.Canvas(root)
   MainCanvas.bind("<Button-1>",DrawOval)
   MainCanvas.bind("<Button-3>",DrawRectangle)
   MainCanvas.pack(fill=Tkinter.BOTH,expand=Tkinter.YES)

   gistButton=Tkinter.Button(MainCanvas,text="Gist",
    command=plot)
   quitButton=Tkinter.Button(MainCanvas,text="Quit",
    command=sys.exit)
   gistButton.pack(side=Tkinter.TOP)
   quitButton.pack(side=Tkinter.BOTTOM)

   root.bind("<Up>",lambda e:MoveButton(Tkinter.TOP))
   root.bind("<Down>",lambda e:MoveButton(Tkinter.BOTTOM))
   root.bind("<Left>",lambda e:MoveButton(Tkinter.LEFT))
   root.bind("<Right>",lambda e:MoveButton(Tkinter.RIGHT))
   root.geometry("300x300") # Set minimum window size

   root.mainloop()

if (__name__=="__main__"):
   main()



#
# Author: Pearu Peterson, March 2002
#
# additions by Travis Oliphant, March 2002
# additions by Eric Jones,      June 2002

__all__ = ['eig','eigvals','lu','svd','svdvals','diagsvd','cholesky','qr',
           'schur','rsf2csf','lu_factor','cho_factor','cho_solve','orth',
           'hessenberg']

from basic import LinAlgError
import basic

from warnings import warn
from lapack import get_lapack_funcs
from blas import get_blas_funcs
from flinalg import get_flinalg_funcs
import calc_lwork
import scipy_base
from scipy_base import asarray_chkfinite, asarray, diag, zeros, ones, \
     dot, transpose
cast = scipy_base.cast
r_ = scipy_base.r_
c_ = scipy_base.c_

_I = cast['F'](1j)
def _make_complex_eigvecs(w,vin,cmplx_tcode):
    v = scipy_base.array(vin,typecode=cmplx_tcode)
    ind = scipy_base.nonzero(scipy_base.not_equal(w.imag,0.0))
    vnew = scipy_base.zeros((v.shape[0],len(ind)>>1),cmplx_tcode)
    vnew.real = scipy_base.take(vin,ind[::2],1)
    vnew.imag = scipy_base.take(vin,ind[1::2],1)
    count = 0
    conj = scipy_base.conjugate
    for i in range(len(ind)/2):
        v[:,ind[2*i]] = vnew[:,count]
        v[:,ind[2*i+1]] = conj(vnew[:,count])
        count += 1
    return v

def _geneig(a1,b,left,right,overwrite_a,overwrite_b):
    b1 = asarray(b)
    overwrite_b = overwrite_b or (b1 is not b and not hasattry(b,'__array__'))
    if len(b1.shape) != 2 or b1.shape[0] != b1.shape[1]:
        raise ValueError, 'expected square matrix'
    ggev, = get_lapack_funcs(('ggev',),(a1,b1))
    cvl,cvr = left,right
    if ggev.module_name[:7] == 'clapack':
        raise NotImplementedError,'calling ggev from %s' % (ggev.module_name)
    res = ggev(a1,b1,lwork=-1)
    lwork = res[-2][0]
    if ggev.prefix in 'cz':
        alpha,beta,vl,vr,work,info = ggev(a1,b1,cvl,cvr,lwork,
                                          overwrite_a,overwrite_b)
        w = alpha / beta
    else:
        alphar,alphai,beta,vl,vr,work,info = ggev(a1,b1,cvl,cvr,lwork,
                                                  overwrite_a,overwrite_b)
        w = (alphar+_I*alphai)/beta
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal ggev'%(-info)
    if info>0: raise LinAlgError,"generalized eig algorithm did not converge"

    only_real = scipy_base.logical_and.reduce(scipy_base.equal(w.imag,0.0))
    if not (ggev.prefix in 'cz' or only_real):
        t = w.typecode()
        if left:
            vl = _make_complex_eigvecs(w, vl, t)
        if right:
            vr = _make_complex_eigvecs(w, vr, t)
    if not (left or right):
        return w
    if left:
        if right:
            return w, vl, vr
        return w, vl
    return w, vr
        
def eig(a,b=None,left=0,right=1,overwrite_a=0,overwrite_b=0):
    """ Solve ordinary and generalized eigenvalue problem
    of a square matrix.

    Inputs:

      a     -- An N x N matrix.
      b     -- An N x N matrix [default is identity(N)].
      left  -- Return left eigenvectors [disabled].
      right -- Return right eigenvectors [enabled].

    Outputs:

      w      -- eigenvalues [left==right==0].
      w,vr   -- w and right eigenvectors [left==0,right=1].
      w,vl   -- w and left eigenvectors [left==1,right==0].
      w,vl,vr  -- [left==right==1].

    Definitions:
      
      a * vr[:,i] = w[i] * b * vr[:,i]

      a^H * vl[:,i] = conjugate(w[i]) * b^H * vl[:,i]

    where a^H denotes transpose(conjugate(a)).
    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    if b is not None:
        b = asarray_chkfinite(b)
        return _geneig(a1,b,left,right,overwrite_a,overwrite_b)
    geev, = get_lapack_funcs(('geev',),(a1,))
    compute_vl,compute_vr=left,right
    if geev.module_name[:7] == 'flapack':
        lwork = calc_lwork.geev(geev.prefix,a1.shape[0],
                                compute_vl,compute_vr)[1]
        if geev.prefix in 'cz':
            w,vl,vr,info = geev(a1,lwork = lwork,
                                compute_vl=compute_vl,
                                compute_vr=compute_vr,
                                overwrite_a=overwrite_a)
        else:
            wr,wi,vl,vr,info = geev(a1,lwork = lwork,
                                    compute_vl=compute_vl,
                                    compute_vr=compute_vr,
                                    overwrite_a=overwrite_a)
            t = {'f':'F','d':'D'}[wr.typecode()]
            w = wr+_I*wi
    else: # 'clapack'
        if geev.prefix in 'cz':
            w,vl,vr,info = geev(a1,
                                compute_vl=compute_vl,
                                compute_vr=compute_vr,
                                overwrite_a=overwrite_a)
        else:
            wr,wi,vl,vr,info = geev(a1,
                                    compute_vl=compute_vl,
                                    compute_vr=compute_vr,
                                    overwrite_a=overwrite_a)
            t = {'f':'F','d':'D'}[wr.typecode()]
            w = wr+_I*wi
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal geev'%(-info)
    if info>0: raise LinAlgError,"eig algorithm did not converge"

    only_real = scipy_base.logical_and.reduce(scipy_base.equal(w.imag,0.0))
    if not (geev.prefix in 'cz' or only_real):
        t = w.typecode()
        if left:
            vl = _make_complex_eigvecs(w, vl, t)
        if right:
            vr = _make_complex_eigvecs(w, vr, t)
    if not (left or right):
        return w
    if left:
        if right:
            return w, vl, vr
        return w, vl
    return w, vr

def eigvals(a,b=None,overwrite_a=0):
    """Return eigenvalues of square matrix."""
    return eig(a,b=b,left=0,right=0,overwrite_a=overwrite_a)
    
def lu_factor(a, overwrite_a=0):
    """Return raw LU decomposition of a matrix and pivots, for use in solving
    a system of linear equations.

    Inputs:

      a  --- an NxN matrix

    Outputs:

      lu --- the lu factorization matrix
      piv --- an array of pivots
    """
    a1 = asarray(a)
    if len(a1.shape) != 2 or (a1.shape[0] != a1.shape[1]):
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    getrf, = get_lapack_funcs(('getrf',),(a1,))
    lu, piv, info = getrf(a,overwrite_a=overwrite_a)
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal getrf (lu_factor)'%(-info)
    if info>0: warn("Diagonal number %d is exactly zero. Singular matrix." % info,
                    RuntimeWarning)
    return lu, piv

def lu_solve(a_lu_pivots,b):
    """Solve a previously factored system.  First input is a tuple (lu, pivots)
    which is the output to lu_factor.  Second input is the right hand side.
    """
    a_lu, pivots = a_lu_pivots
    a_lu = asarray_chkfinite(a_lu)
    pivots = asarray_chkfinite(pivots)
    b = asarray_chkfinite(b)
    _assert_squareness(a_lu)
    
    getrs, = get_lapack_funcs(('getrs',),(a,))
    b, info = getrs(a_lu,pivots,b)    
    if info < 0:
        msg = "Argument %d to lapack's ?getrs() has an illegal value." % info
        raise TypeError, msg
    if info > 0:
        msg = "Unknown error occured int ?getrs(): error code = %d" % info
        raise TypeError, msg
    return b
    
    
def lu(a,permute_l=0,overwrite_a=0):
    """Return LU decompostion of a matrix.

    Inputs:

      a     -- An M x N matrix.
      permute_l  -- Perform matrix multiplication p * l [disabled].

    Outputs:

      p,l,u  -- LU decomposition matrices of a [permute_l=0]
      pl,u   -- LU decomposition matrices of a [permute_l=1]

    Definitions:
      
      a = p * l * u    [permute_l=0]
      a = pl * u       [permute_l=1]

      p   -  An M x M permutation matrix
      l   -  An M x K lower triangular or trapezoidal matrix
             with unit-diagonal
      u   -  An K x N upper triangular or trapezoidal matrix
             K = min(M,N)
    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2:
        raise ValueError, 'expected matrix'
    m,n = a1.shape
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    flu, = get_flinalg_funcs(('lu',),(a1,))
    p,l,u,info = flu(a1,permute_l=permute_l,overwrite_a = overwrite_a)
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal lu.getrf'%(-info)
    if permute_l:
        return l,u
    return p,l,u

def svd(a,compute_uv=1,overwrite_a=0):
    """Compute singular value decomposition (SVD) of matrix a.

    Description:

      Singular value decomposition of a matrix a is
        a = u * sigma * v^H,
      where v^H denotes conjugate(transpose(v)), u,v are unitary
      matrices, sigma is zero matrix with a main diagonal containing
      real non-negative singular values of the matrix a.

    Inputs:

      a -- An M x N matrix.
      compute_uv -- If zero, then only the vector of singular values
                    is returned.

    Outputs:

      u -- An M x M unitary matrix [compute_uv=1].
      s -- An min(M,N) vector of singular values in descending order,
           sigma = diagsvd(s).
      vh -- An N x N unitary matrix [compute_uv=1], vh = v^H.

    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2:
        raise ValueError, 'expected matrix'
    m,n = a1.shape
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    gesdd, = get_lapack_funcs(('gesdd',),(a1,))
    if gesdd.module_name[:7] == 'flapack':
        lwork = calc_lwork.gesdd(gesdd.prefix,m,n,compute_uv)[1]
        u,s,v,info = gesdd(a1,compute_uv = compute_uv, lwork = lwork,
                      overwrite_a = overwrite_a)
    else: # 'clapack'
        raise NotImplementedError,'calling gesdd from %s' % (gesdd.module_name)
    if info>0: raise LinAlgError, "SVD did not converge"
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal gesdd'%(-info)
    if compute_uv:
        return u,s,v
    else:
        return s

def svdvals(a,overwrite_a=0):
    """Return singular values of a matrix."""
    return svd(a,compute_uv=0,overwrite_a=overwrite_a)

def diagsvd(s,M,N):
    """Return sigma from singular values and original size M,N."""
    part = diag(s)
    typ = part.typecode()
    MorN = len(s)
    if MorN == M:
        return c_[part,zeros((M,N-M),typ)]
    elif MorN == N:
        return r_[part,zeros((M-N,N),typ)]
    else:
        raise ValueError, "Length of s must be M or N."

def cholesky(a,lower=0,overwrite_a=0):
    """Compute Cholesky decomposition of matrix.

    Description:

      For a hermitian positive-definite matrix a return the
      upper-triangular (or lower-triangular if lower==1) matrix,
      u such that u^H * u = a (or l * l^H = a).

    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    potrf, = get_lapack_funcs(('potrf',),(a1,))
    c,info = potrf(a1,lower=lower,overwrite_a=overwrite_a,clean=1)
    if info>0: raise LinAlgError, "matrix not positive definite"
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal potrf'%(-info)
    return c

def cho_factor(a, lower=0, overwrite_a=0):
    """ Compute Cholesky decomposition of matrix and return an object
    to be used for solving a linear system using cho_solve.
    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    potrf, = get_lapack_funcs(('potrf',),(a1,))
    c,info = potrf(a1,lower=lower,overwrite_a=overwrite_a,clean=0)
    if info>0: raise LinAlgError, "matrix not positive definite"
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal potrf'%(-info)
    return c, lower

def cho_solve(clow, b):
    """Solve a previously factored symmetric system of equations.
    First input is a tuple (LorU, lower) which is the output to cho_factor.
    Second input is the right-hand side.
    """
    c, lower = clow
    c = asarray_chkfinite(c)
    _assert_squareness(c)
    b = asarray_chkfinite(b)
    potrs, = get_lapack_funcs(('potrs',),(c,))
    b, info = potrs(c,b,lower)
    if info < 0:
        msg = "Argument %d to lapack's ?potrs() has an illegal value." % info
        raise TypeError, msg
    if info > 0:
        msg = "Unknown error occured int ?potrs(): error code = %d" % info
        raise TypeError, msg
    return b


def qr(a,overwrite_a=0,lwork=None):
    """QR decomposition of an M x N matrix a.

    Description:

      Find a unitary matrix, q, and an upper-trapezoidal matrix r
      such that q * r = a

    Inputs:

      a -- the matrix
      overwrite_a=0 -- if non-zero then discard the contents of a,
                     i.e. a is used as a work array if possible.

      lwork=None -- >= shape(a)[1]. If None (or -1) compute optimal
                    work array size. 

    Outputs:
    
      q, r -- matrices such that q * r = a

    """
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2:
        raise ValueError, 'expected matrix'
    M,N = a1.shape
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    geqrf, = get_lapack_funcs(('geqrf',),(a1,))
    if lwork is None or lwork == -1:
        # get optimal work array
        qr,tau,work,info = geqrf(a1,lwork=-1,overwrite_a=1)
        lwork = work[0]
    qr,tau,work,info = geqrf(a1,lwork=lwork,overwrite_a=overwrite_a)    
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal geqrf'%(-info)
    gemm, = get_blas_funcs(('gemm',),(qr,))
    t = qr.typecode()
    R = basic.triu(qr)
    Q = scipy_base.identity(M,typecode=t)
    ident = scipy_base.identity(M,typecode=t)
    zeros = scipy_base.zeros
    for i in range(min(M,N)):
        v = zeros((M,),t)
        v[i] = 1
        v[i+1:M] = qr[i+1:M,i]
        H = gemm(-tau[i],v,v,1+0j,ident,trans_b=2)
        Q = gemm(1,Q,H)
    return Q, R

_double_precision = ['i','l','d']

def schur(a,output='real',lwork=None,overwrite_a=0):
    """Compute Schur decomposition of matrix a. 

    Description:

      Return T, Z such that a = Z * T * (Z**H) where Z is a
      unitary matrix and T is either upper-triangular or quasi-upper
      triangular for output='real'
    """
    if not output in ['real','complex','r','c']:
        raise ValueError, "argument must be 'real', or 'complex'"
    a1 = asarray_chkfinite(a)
    if len(a1.shape) != 2 or (a1.shape[0] != a1.shape[1]):
        raise ValueError, 'expected square matrix'
    N = a1.shape[0]
    typ = a1.typecode()
    if output in ['complex','c'] and typ not in ['F','D']:
        if typ in _double_precision:
            a1 = a1.astype('D')
            typ = 'D'
        else:
            a1 = a1.astype('F')
            typ = 'F'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    gees, = get_lapack_funcs(('gees',),(a1,))
    if lwork is None or lwork == -1:
        # get optimal work array
        result = gees(lambda x: None,a,lwork=-1)
        lwork = result[-2][0]
    result = gees(lambda x: None,a,lwork=result[-2][0],overwrite_a=overwrite_a)
    info = result[-1]
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal gees'%(-info)
    elif info>0: raise LinAlgError, "Schur form not found.  Possibly ill-conditioned."
    return result[0], result[-3]

eps = scipy_base.limits.double_epsilon
feps = scipy_base.limits.float_epsilon

_array_kind = {'1':0, 's':0, 'b': 0, 'i':0, 'l': 0, 'f': 0, 'd': 0, 'F': 1, 'D': 1}
_array_precision = {'i': 1, 'l': 1, 'f': 0, 'd': 1, 'F': 0, 'D': 1}
_array_type = [['f', 'd'], ['F', 'D']]
def _commonType(*arrays):
    kind = 0
    precision = 0
    for a in arrays:
        t = a.typecode()
        kind = max(kind, _array_kind[t])
        precision = max(precision, _array_precision[t])
    return _array_type[kind][precision]

def _castCopy(type, *arrays):
    cast_arrays = ()
    for a in arrays:
        if a.typecode() == type:
            cast_arrays = cast_arrays + (a.copy(),)
        else:
            cast_arrays = cast_arrays + (a.astype(type),)
    if len(cast_arrays) == 1:
            return cast_arrays[0]
    else:
        return cast_arrays

def _assert_squareness(*arrays):
    for a in arrays:
        if max(a.shape) != min(a.shape):
            raise LinAlgError, 'Array must be square'

def rsf2csf(T, Z):
    """Convert real schur form to complex schur form.

    Description:

      If A is a real-valued matrix, then the real schur form is
      quasi-upper triangular.  2x2 blocks extrude from the main-diagonal
      corresponding to any complex-valued eigenvalues.

      This function converts this real schur form to a complex schur form
      which is upper triangular.
    """
    Z,T = map(asarray_chkfinite, (Z,T))
    if len(Z.shape) !=2 or Z.shape[0] != Z.shape[1]:
        raise ValueError, "matrix must be square."
    if len(T.shape) !=2 or T.shape[0] != T.shape[1]:
        raise ValueError, "matrix must be square."
    if T.shape[0] != Z.shape[0]:
        raise ValueError, "matrices must be same dimension."
    N = T.shape[0]
    arr = scipy_base.array    
    t = _commonType(Z, T, arr([3.0],'F'))
    Z, T = _castCopy(t, Z, T)
    conj = scipy_base.conj
    dot = scipy_base.dot
    r_ = scipy_base.r_
    transp = scipy_base.transpose
    for m in range(N-1,0,-1):
        if abs(T[m,m-1]) > eps*(abs(T[m-1,m-1]) + abs(T[m,m])):
            k = slice(m-1,m+1)
            mu = eigvals(T[k,k]) - T[m,m]
            r = basic.norm([mu[0], T[m,m-1]])
            c = mu[0] / r
            s = T[m,m-1] / r
            G = r_[arr([[conj(c),s]],typecode=t),arr([[-s,c]],typecode=t)]
            Gc = conj(transp(G))
            j = slice(m-1,N)
            T[k,j] = dot(G,T[k,j])
            i = slice(0,m+1)
            T[i,k] = dot(T[i,k], Gc)
            i = slice(0,N)
            Z[i,k] = dot(Z[i,k], Gc)
        T[m,m-1] = 0.0;
    return T, Z


# Orthonormal decomposition

def orth(A):
    """Return an orthonormal basis for the range of A using svd"""
    u,s,vh = svd(A)
    M,N = A.shape
    tol = max(M,N)*scipy_base.amax(s)*eps
    num = scipy_base.sum(s > tol)
    Q = u[:,:num]
    return Q

def hessenberg(a,calc_q=0,overwrite_a=0):
    """ Compute Hessenberg form of a matrix.

    Inputs:

      a -- the matrix
      calc_q -- if non-zero then calculate unitary similarity
                transformation matrix q.
      overwrite_a=0 -- if non-zero then discard the contents of a,
                     i.e. a is used as a work array if possible.

    Outputs:

      h    -- Hessenberg form of a                [calc_q=0]
      h, q -- matrices such that a = q * h * q^T  [calc_q=1]

    """
    a1 = asarray(a)
    if len(a1.shape) != 2 or (a1.shape[0] != a1.shape[1]):
        raise ValueError, 'expected square matrix'
    overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
    gehrd,gebal = get_lapack_funcs(('gehrd','gebal'),(a1,))
    ba,lo,hi,pivscale,info = gebal(a,permute=1,overwrite_a = overwrite_a)
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal gebal (hessenberg)'%(-info)
    n = len(a1)
    lwork = calc_lwork.gehrd(gehrd.prefix,n,lo,hi)
    hq,tau,info = gehrd(ba,lo=lo,hi=hi,lwork=lwork,overwrite_a=1)
    if info<0: raise ValueError,\
       'illegal value in %-th argument of internal gehrd (hessenberg)'%(-info)

    if not calc_q:
        for i in range(lo,hi):
            hq[i+2:hi+1,i] = 0.0
        return hq

    # XXX: Use ORGHR routines to compute q.
    ger,gemm = get_blas_funcs(('ger','gemm'),(hq,))
    typecode = hq.typecode()
    q = None
    for i in range(lo,hi):
        if tau[i]==0.0:
            continue
        v = zeros(n,typecode=typecode)
        v[i+1] = 1.0
        v[i+2:hi+1] = hq[i+2:hi+1,i]
        hq[i+2:hi+1,i] = 0.0
        h = ger(-tau[i],v,v,a=diag(ones(n,typecode=typecode)),overwrite_a=1)
        if q is None:
            q = h
        else:
            q = gemm(1.0,q,h)
    if q is None:
        q = diag(ones(n,typecode=typecode))
    return hq,q



#!/usr/bin/env python
# Created by Pearu Peterson, September 2002
""" Test functions for fftpack.basic module
"""
__usage__ = """
Build fftpack:
  python setup_fftpack.py build
Run tests if scipy is installed:
  python -c 'import scipy;scipy.fftpack.test(<level>)'
Run tests if fftpack is not installed:
  python tests/test_basic.py [<level>]
"""
import sys
from scipy_test.testing import *
set_package_path()
from fftpack import ifft,fft,fftn,ifftn,rfft,irfft
from fftpack import _fftpack as fftpack
del sys.path[0]


import Numeric
from Numeric import arange, add, array

from scipy_test.testing import rand
def random(size):
    return rand(*size)

import unittest

def get_mat(n):
    data = arange(n)
    data = add.outer(data,data)
    return data

def direct_dft(x):
    x = Numeric.asarray(x)
    n = len(x)
    y = Numeric.zeros(n,'D')
    w = -Numeric.arange(n)*(2j*Numeric.pi/n)
    for i in range(n):
        y[i] = Numeric.dot(Numeric.exp(i*w),x)
    return y

def direct_idft(x):
    x = Numeric.asarray(x)
    n = len(x)
    y = Numeric.zeros(n,'D')
    w = Numeric.arange(n)*(2j*Numeric.pi/n)
    for i in range(n):
        y[i] = Numeric.dot(Numeric.exp(i*w),x)/n
    return y

def direct_dftn(x):
    x = Numeric.asarray(x)
    for axis in range(len(x.shape)):
        x = fft(x,axis=axis)
    return x

def direct_idftn(x):
    x = Numeric.asarray(x)
    for axis in range(len(x.shape)):
        x = ifft(x,axis=axis)
    return x

def direct_rdft(x):
    x = Numeric.asarray(x)
    n = len(x)
    w = -Numeric.arange(n)*(2j*Numeric.pi/n)
    r = Numeric.zeros(n,'d')
    for i in range(n/2+1):
        y = Numeric.dot(Numeric.exp(i*w),x)
        if i:
            r[2*i-1] = y.real
            if 2*i<n:
                r[2*i] = y.imag
        else:
            r[0] = y.real
    return r

def direct_irdft(x):
    x = Numeric.asarray(x)
    n = len(x)
    x1 = Numeric.zeros(n,'D')
    for i in range(n/2+1):
        if i:
            if 2*i<n:
                x1[i] = x[2*i-1] + 1j* x[2*i]
                x1[n-i] = x[2*i-1] - 1j* x[2*i]
            else:
                x1[i] = x[2*i-1]
        else:
            x1[0] = x[0]
    return direct_idft(x1).real

class test_fft(ScipyTestCase):

    def check_definition(self):
        x = [1,2,3,4+1j,1,2,3,4+2j]
        y = fft(x)
        y1 = direct_dft(x)
        assert_array_almost_equal(y,y1)
        x = [1,2,3,4+0j,5]
        assert_array_almost_equal(fft(x),direct_dft(x))

    def check_n_argument_real(self):
        x1 = [1,2,3,4]
        x2 =  [1,2,3,4]
        y = fft([x1,x2],n=4)
        assert_equal(y.shape,(2,4))
        assert_array_almost_equal(y[0],direct_dft(x1))
        assert_array_almost_equal(y[1],direct_dft(x2))

    def check_n_argument_complex(self):
        x1 = [1,2,3,4+1j]
        x2 =  [1,2,3,4+1j]
        y = fft([x1,x2],n=4)
        assert_equal(y.shape,(2,4))
        assert_array_almost_equal(y[0],direct_dft(x1))
        assert_array_almost_equal(y[1],direct_dft(x2))

    def check_djbfft(self):
        try:
            from FFT import fft as fft2
        except ImportError:
            print 'Skipping check_djbfft (failed to import FFT)'
            return
        for i in range(2,14):
            n = 2**i
            x = range(n)
            y = fftpack.zfft(x)
            y2 = fft2(x)
            assert_array_almost_equal(y,y2)
            y = fftpack.zrfft(x)
            assert_array_almost_equal(y,y2)

    def bench_random(self,level=5):
        try:
            from FFT import fft as FFT_fft
        except ImportError:
            FFT_fft = None
        print
        print '                 Fast Fourier Transform'
        print '================================================='
        print '      |    real input     |   complex input    '
        print '-------------------------------------------------'
        print ' size |  scipy  | Numeric |  scipy  | Numeric '
        print '-------------------------------------------------'
        for size,repeat in [(100,7000),(1000,2000),
                            (256,10000),
                            (512,10000),
                            (1024,1000),
                            (2048,1000),
                            (2048*2,500),
                            (2048*4,500),
                            ]:
            print '%5s' % size,
            sys.stdout.flush()

            for x in [random([size]).astype('d'),
                      random([size]).astype('D')+random([size]).astype('D')*1j
                      ]:
                if size > 500: y = fft(x)
                else: y = direct_dft(x)                
                assert_array_almost_equal(fft(x),y)
                print '|%8.2f' % self.measure('fft(x)',repeat),
                sys.stdout.flush()
                
                if FFT_fft is not None:
                    assert_array_almost_equal(FFT_fft(x),y)
                    print '|%8.2f' % self.measure('FFT_fft(x)',repeat),
                else:
                    print '|  N/A  ',
                sys.stdout.flush()

            print ' (secs for %s calls)' % (repeat)


class test_ifft(ScipyTestCase):

    def check_definition(self):
        x = [1,2,3,4+1j,1,2,3,4+2j]
        y = ifft(x)
        y1 = direct_idft(x)
        assert_array_almost_equal(y,y1)
        x = [1,2,3,4+0j,5]
        assert_array_almost_equal(ifft(x),direct_idft(x))
        x = [1,2,3,4,1,2,3,4]
        y = ifft(x)
        y1 = direct_idft(x)
        assert_array_almost_equal(y,y1)
        x = [1,2,3,4,5]
        assert_array_almost_equal(ifft(x),direct_idft(x))

    def check_djbfft(self):
        try:
            from FFT import inverse_fft as ifft2
        except ImportError:
            return
        for i in range(2,14):
            n = 2**i
            x = range(n)
            y = fftpack.zfft(x,direction=-1)
            y2 = ifft2(x)
            assert_array_almost_equal(y,y2)
            y = fftpack.zrfft(x,direction=-1)
            assert_array_almost_equal(y,y2)

    def check_random_complex(self):
        for size in [1,51,111,100,200,64,128,256,1024]:
            x = random([size]).astype('D')
            x = random([size]).astype('D') +1j*x
            assert_array_almost_equal (ifft(fft(x)),x)
            assert_array_almost_equal (fft(ifft(x)),x)

    def check_random_real(self):
        for size in [1,51,111,100,200,64,128,256,1024]:
            x = random([size]).astype('d')
            assert_array_almost_equal (ifft(fft(x)),x)
            assert_array_almost_equal (fft(ifft(x)),x)

    def bench_random(self,level=5):
        try:
            from FFT import inverse_fft as FFT_ifft
        except ImportError:
            FFT_ifft = None

        print
        print '       Inverse Fast Fourier Transform'
        print '==============================================='
        print '      |     real input    |    complex input   '
        print '-----------------------------------------------'
        print ' size |  scipy  | Numeric |  scipy  | Numeric  '
        print '-----------------------------------------------'
        for size,repeat in [(100,7000),(1000,2000),
                            (256,10000),
                            (512,10000),
                            (1024,1000),
                            (2048,1000),
                            (2048*2,500),
                            (2048*4,500),
                            ]:
            print '%5s' % size,
            sys.stdout.flush()

            for x in [random([size]).astype('d'),
                      random([size]).astype('D')+random([size]).astype('D')*1j
                      ]:
                if size > 500: y = ifft(x)
                else: y = direct_idft(x)                
                assert_array_almost_equal(ifft(x),y)
                print '|%8.2f' % self.measure('ifft(x)',repeat),
                sys.stdout.flush()

                if FFT_ifft is not None:
                    assert_array_almost_equal(FFT_ifft(x),y)
                    print '|%8.2f' % self.measure('FFT_ifft(x)',repeat),
                else:
                    print '|  N/A  ',
                sys.stdout.flush()

            print ' (secs for %s calls)' % (repeat)


class test_rfft(ScipyTestCase):

    def check_definition(self):
        x = [1,2,3,4,1,2,3,4]
        y = rfft(x)
        y1 = direct_rdft(x)
        assert_array_almost_equal(y,y1)
        x = [1,2,3,4,1,2,3,4,5]
        y = rfft(x)
        y1 = direct_rdft(x)
        assert_array_almost_equal(y,y1)

    def check_djbfft(self):
        try:
            from FFT import fft as fft2
        except ImportError,errmess:
            print 'Skipping check_djbfft (failed to import FFT: %s)' % errmess
            return
        for i in range(2,14):
            n = 2**i
            x = range(n)
            y2 = fft2(x)
            y1 = Numeric.zeros((n,),'d')
            y1[0] = y2[0].real
            y1[-1] = y2[n/2].real
            for k in range(1,n/2):
                y1[2*k-1] = y2[k].real
                y1[2*k] = y2[k].imag
            y = fftpack.drfft(x)
            assert_array_almost_equal(y,y1)

    def bench_random(self,level=5):
        try:
            from FFT import real_fft as FFT_rfft
        except ImportError:
            FFT_rfft = None

        print
        print 'Fast Fourier Transform (real data)'
        print '=================================='
        print ' size |  scipy  | Numeric  '
        print '----------------------------------'
        for size,repeat in [(100,7000),(1000,2000),
                            (256,10000),
                            (512,10000),
                            (1024,1000),
                            (2048,1000),
                            (2048*2,500),
                            (2048*4,500),
                            ]:
            print '%5s' % size,
            sys.stdout.flush()

            x = random([size]).astype('d')
            print '|%8.2f' % self.measure('rfft(x)',repeat),
            sys.stdout.flush()
            
            if FFT_rfft is not None:
                print '|%8.2f' % self.measure('FFT_rfft(x)',repeat),
            else:
                print '|  N/A  ',
            sys.stdout.flush()

            print ' (secs for %s calls)' % (repeat)


class test_irfft(ScipyTestCase):

    def check_definition(self):
        x = [1,2,3,4,1,2,3,4]
        x1 = [1,2+3j,4+1j,2+3j,4,2-3j,4-1j,2-3j]
        y = irfft(x)
        y1 = direct_irdft(x)
        assert_array_almost_equal(y,y1)
        assert_array_almost_equal(y,ifft(x1))
        x = [1,2,3,4,1,2,3,4,5]
        x1 = [1,2+3j,4+1j,2+3j,4+5j,4-5j,2-3j,4-1j,2-3j]
        y = irfft(x)
        y1 = direct_irdft(x)
        assert_array_almost_equal(y,y1)
        assert_array_almost_equal(y,ifft(x1))

    def check_djbfft(self):
        try:
            from FFT import inverse_fft as ifft2
        except ImportError:
            print 'Skipping check_djbfft (failed to import FFT)'
            return
        for i in range(2,14):
            n = 2**i
            x = range(n)
            x1 = Numeric.zeros((n,),'D')
            x1[0] = x[0]
            for k in range(1,n/2):
                x1[k] = x[2*k-1]+1j*x[2*k]
                x1[n-k] = x[2*k-1]-1j*x[2*k]
            x1[n/2] = x[-1]
            y1 = ifft2(x1)
            y = fftpack.drfft(x,direction=-1)
            assert_array_almost_equal(y,y1)

    def check_random_real(self):
        for size in [1,51,111,100,200,64,128,256,1024]:
            x = random([size]).astype('d')
            assert_array_almost_equal (irfft(rfft(x)),x)
            assert_array_almost_equal (rfft(irfft(x)),x)

    def bench_random(self,level=5):
        try:
            from FFT import inverse_real_fft as FFT_irfft
        except ImportError:
            FFT_irfft = None

        print
        print 'Inverse Fast Fourier Transform (real data)'
        print '=================================='
        print ' size |  scipy  | Numeric  '
        print '----------------------------------'
        for size,repeat in [(100,7000),(1000,2000),
                            (256,10000),
                            (512,10000),
                            (1024,1000),
                            (2048,1000),
                            (2048*2,500),
                            (2048*4,500),
                            ]:
            print '%5s' % size,
            sys.stdout.flush()

            x = random([size]).astype('d')
            x1 = Numeric.zeros(size/2+1,'D')
            x1[0] = x[0]
            for i in range(1,size/2):
                x1[i] = x[2*i-1] + 1j * x[2*i]
            if not size%2:
                x1[-1] = x[-1]
            y = irfft(x)

            print '|%8.2f' % self.measure('irfft(x)',repeat),
            sys.stdout.flush()
            
            if FFT_irfft is not None:
                assert_array_almost_equal(FFT_irfft(x1,size),y)
                print '|%8.2f' % self.measure('FFT_irfft(x1,size)',repeat),
            else:
                print '|  N/A  ',
            sys.stdout.flush()

            print ' (secs for %s calls)' % (repeat)

class test_fftn(ScipyTestCase):

    def check_definition(self):
        x = [[1,2,3],[4,5,6],[7,8,9]]
        y = fftn(x)
        assert_array_almost_equal(y,direct_dftn(x))
        x = random((20,26))
        assert_array_almost_equal(fftn(x),direct_dftn(x))
        x = random((5,4,3,20))
        assert_array_almost_equal(fftn(x),direct_dftn(x))

    def check_axes_argument(self):
        #plane == ji_plane, x== kji_space
        plane1 = [[1,2,3],[4,5,6],[7,8,9]]
        plane2 = [[10,11,12],[13,14,15],[16,17,18]]
        plane3 = [[19,20,21],[22,23,24],[25,26,27]]
        ki_plane1 = [[1,2,3],[10,11,12],[19,20,21]]
        ki_plane2 = [[4,5,6],[13,14,15],[22,23,24]]
        ki_plane3 = [[7,8,9],[16,17,18],[25,26,27]]
        jk_plane1 = [[1,10,19],[4,13,22],[7,16,25]]
        jk_plane2 = [[2,11,20],[5,14,23],[8,17,26]]
        jk_plane3 = [[3,12,21],[6,15,24],[9,18,27]]
        kj_plane1 = [[1,4,7],[10,13,16],[19,22,25]]
        kj_plane2 = [[2,5,8],[11,14,17],[20,23,26]]
        kj_plane3 = [[3,6,9],[12,15,18],[21,24,27]]
        ij_plane1 = [[1,4,7],[2,5,8],[3,6,9]]
        ij_plane2 = [[10,13,16],[11,14,17],[12,15,18]]
        ij_plane3 = [[19,22,25],[20,23,26],[21,24,27]]
        ik_plane1 = [[1,10,19],[2,11,20],[3,12,21]]
        ik_plane2 = [[4,13,22],[5,14,23],[6,15,24]]
        ik_plane3 = [[7,16,25],[8,17,26],[9,18,27]]
        ijk_space = [jk_plane1,jk_plane2,jk_plane3]
        ikj_space = [kj_plane1,kj_plane2,kj_plane3]
        jik_space = [ik_plane1,ik_plane2,ik_plane3]
        jki_space = [ki_plane1,ki_plane2,ki_plane3]
        kij_space = [ij_plane1,ij_plane2,ij_plane3]
        x = Numeric.array([plane1,plane2,plane3])

        assert_array_almost_equal(fftn(x),fftn(x,axes=(-3,-2,-1))) # kji_space
        assert_array_almost_equal(fftn(x),fftn(x,axes=(0,1,2)))
        y = fftn(x,axes=(2,1,0)) # ijk_space
        assert_array_almost_equal(Numeric.swapaxes(y,-1,-3),fftn(ijk_space))
        y = fftn(x,axes=(2,0,1)) # ikj_space
        assert_array_almost_equal(Numeric.swapaxes(Numeric.swapaxes(y,-1,-3),
                                                   -1,-2)
                                  ,fftn(ikj_space))
        y = fftn(x,axes=(1,2,0)) # jik_space
        assert_array_almost_equal(Numeric.swapaxes(Numeric.swapaxes(y,-1,-3),
                                                   -3,-2)
                                  ,fftn(jik_space))
        y = fftn(x,axes=(1,0,2)) # jki_space
        assert_array_almost_equal(Numeric.swapaxes(y,-2,-3),fftn(jki_space))
        y = fftn(x,axes=(0,2,1)) # kij_space
        assert_array_almost_equal(Numeric.swapaxes(y,-2,-1),
                                  fftn(kij_space))
        
        y = fftn(x,axes=(-2,-1)) # ji_plane
        assert_array_almost_equal(fftn(plane1),y[0])
        assert_array_almost_equal(fftn(plane2),y[1])
        assert_array_almost_equal(fftn(plane3),y[2])
        y = fftn(x,axes=(1,2)) # ji_plane
        assert_array_almost_equal(fftn(plane1),y[0])
        assert_array_almost_equal(fftn(plane2),y[1])
        assert_array_almost_equal(fftn(plane3),y[2])
        y = fftn(x,axes=(-3,-2)) # kj_plane
        assert_array_almost_equal(fftn(x[:,:,0]),y[:,:,0])
        assert_array_almost_equal(fftn(x[:,:,1]),y[:,:,1])
        assert_array_almost_equal(fftn(x[:,:,2]),y[:,:,2])
        y = fftn(x,axes=(-3,-1)) # ki_plane
        assert_array_almost_equal(fftn(x[:,0,:]),y[:,0,:])
        assert_array_almost_equal(fftn(x[:,1,:]),y[:,1,:])
        assert_array_almost_equal(fftn(x[:,2,:]),y[:,2,:])
        y = fftn(x,axes=(-1,-2)) # ij_plane
        assert_array_almost_equal(fftn(ij_plane1),Numeric.swapaxes(y[0],-2,-1))
        assert_array_almost_equal(fftn(ij_plane2),Numeric.swapaxes(y[1],-2,-1))
        assert_array_almost_equal(fftn(ij_plane3),Numeric.swapaxes(y[2],-2,-1))
        y = fftn(x,axes=(-1,-3)) # ik_plane
        assert_array_almost_equal(fftn(ik_plane1),Numeric.swapaxes(y[:,0,:],-1,-2))
        assert_array_almost_equal(fftn(ik_plane2),Numeric.swapaxes(y[:,1,:],-1,-2))
        assert_array_almost_equal(fftn(ik_plane3),Numeric.swapaxes(y[:,2,:],-1,-2))
        y = fftn(x,axes=(-2,-3)) # jk_plane
        assert_array_almost_equal(fftn(jk_plane1),Numeric.swapaxes(y[:,:,0],-1,-2))
        assert_array_almost_equal(fftn(jk_plane2),Numeric.swapaxes(y[:,:,1],-1,-2))
        assert_array_almost_equal(fftn(jk_plane3),Numeric.swapaxes(y[:,:,2],-1,-2))

        y = fftn(x,axes=(-1,)) # i_line
        for i in range(3):
            for j in range(3):
                assert_array_almost_equal(fft(x[i,j,:]),y[i,j,:])
        y = fftn(x,axes=(-2,)) # j_line
        for i in range(3):
            for j in range(3):
                assert_array_almost_equal(fft(x[i,:,j]),y[i,:,j])
        y = fftn(x,axes=(0,)) # k_line
        for i in range(3):
            for j in range(3):
                assert_array_almost_equal(fft(x[:,i,j]),y[:,i,j])

        y = fftn(x,axes=()) # point
        assert_array_almost_equal(y,x)

    def check_shape_argument(self):
        small_x = [[1,2,3],[4,5,6]]
        large_x1 = [[1,2,3,0],[4,5,6,0],[0,0,0,0],[0,0,0,0]]
        y = fftn(small_x,shape=(4,4))
        assert_array_almost_equal (y,fftn(large_x1))
        y = fftn(small_x,shape=(3,4))
        assert_array_almost_equal (y,fftn(large_x1[:-1]))

    def check_shape_axes_argument(self):
        small_x = [[1,2,3],[4,5,6],[7,8,9]]
        large_x1 = Numeric.array([[1,2,3,0],
                                  [4,5,6,0],
                                  [7,8,9,0],
                                  [0,0,0,0]])
        y = fftn(small_x,shape=(4,4),axes=(-1,))
        for i in range(4):
            assert_array_almost_equal (y[i],fft(large_x1[i]))
        y = fftn(small_x,shape=(4,4),axes=(-2,))
        for i in range(4):
            assert_array_almost_equal (y[:,i],fft(large_x1[:,i]))
        y = fftn(small_x,shape=(4,4),axes=(-2,-1))
        assert_array_almost_equal (y,fftn(large_x1))
        y = fftn(small_x,shape=(4,4),axes=(-1,-2))
        assert_array_almost_equal (y,Numeric.swapaxes(\
            fftn(Numeric.swapaxes(large_x1,-1,-2)),-1,-2))

    def bench_random(self,level=5):
        try:
            from FFT import fftnd as FFT_fftn
        except ImportError:
            FFT_fftn = None

        print
        print '    Multi-dimensional Fast Fourier Transform'
        print '==================================================='
        print '          |    real input     |   complex input    '
        print '---------------------------------------------------'
        print '   size   |  scipy  | Numeric |  scipy  |  Numeric '
        print '---------------------------------------------------'
        for size,repeat in [((100,100),100),((1000,100),7),
                            ((256,256),10),
                            ((512,512),3),
                            ]:
            print '%9s' % ('%sx%s'%size),
            sys.stdout.flush()

            for x in [random(size).astype('d'),
                      random(size).astype('D')+random(size).astype('D')*1j
                      ]:
                y = fftn(x)
                #if size > 500: y = fftn(x)
                #else: y = direct_dft(x)                
                assert_array_almost_equal(fftn(x),y)
                print '|%8.2f' % self.measure('fftn(x)',repeat),
                sys.stdout.flush()
                
                if FFT_fftn is not None:
                    assert_array_almost_equal(FFT_fftn(x),y)
                    print '|%8.2f' % self.measure('FFT_fftn(x)',repeat),
                else:
                    print '|  N/A  ',
                sys.stdout.flush()

            print ' (secs for %s calls)' % (repeat)


class test_ifftn(ScipyTestCase):

    def check_definition(self):
        x = [[1,2,3],[4,5,6],[7,8,9]]
        y = ifftn(x)
        assert_array_almost_equal(y,direct_idftn(x))
        x = random((20,26))
        assert_array_almost_equal(ifftn(x),direct_idftn(x))
        x = random((5,4,3,20))
        assert_array_almost_equal(ifftn(x),direct_idftn(x))

    def check_random_complex(self):
        for size in [1,2,51,32,64,92]:
            x = random([size,size]) + 1j*random([size,size])
            assert_array_almost_equal (ifftn(fftn(x)),x)
            assert_array_almost_equal (fftn(ifftn(x)),x)

if __name__ == "__main__":
    ScipyTest('fftpack.basic').run()



#
# Author: Travis Oliphant, March 2002
#

__all__ = ['expm','expm2','expm3','cosm','sinm','tanm','coshm','sinhm',
           'tanhm','logm','funm','signm','sqrtm']

from scipy_base import asarray, Inf, dot, floor, log2, eye, diag, exp, \
     product, logical_not, ravel, transpose, conjugate, \
     cast, log, ogrid, isfinite, imag, real, absolute, amax, sign, \
     isfinite, sqrt
from Matrix import Matrix as mat
import scipy_base as sb
from basic import solve, LinAlgError, inv, norm, triu, all_mat
from decomp import eig, schur, rsf2csf, orth, eigvals, svd

eps = sb.limits.double_epsilon
feps = sb.limits.float_epsilon

def expm(A,q=7):
    """Compute the matrix exponential using Pade approximation of order q.
    """
    A = asarray(A)
    ss = A.spacesaver()
    if A.typecode() in ['f', 'F']:
        A.savespace(1)
    else:
        A.savespace(0)
 
    # Scale A so that norm is < 1/2
    val = log2(norm(A,Inf))
    e = int(floor(val))
    j = max(0,e+1)
    A = A / 2.0**j
 
    # Pade Approximation for exp(A)
    X = A
    c = 1.0/2
    N = eye(*A.shape) + c*A
    D = eye(*A.shape) - c*A
    for k in range(2,q+1):
        c = c * (q-k+1) / (k*(2*q-k+1))
        X = dot(A,X)
        cX = c*X
        N = N + cX
        if not k % 2:
            D = D + cX;
        else:
            D = D - cX;
    F = solve(D,N)
    for k in range(1,j+1):
        F = dot(F,F)
    A.savespace(ss)
    return F

def expm2(A):
    """Compute the matrix exponential using eigenvalue decomposition.
    """
    A = asarray(A)
    t = A.typecode()
    if t not in ['f','F','d','D']:
        A = A.astype('d')
        t = 'd'    
    s,vr = eig(A)
    vri = inv(vr)
    return dot(dot(vr,diag(exp(s))),vri).astype(t)

def expm3(A,q=20):
    """Compute the matrix exponential using a Taylor series.of order q.
    """
    A = asarray(A)
    t = A.typecode()
    if t not in ['f','F','d','D']:
        A = A.astype('d')
        t = 'd'
    A = mat(A)
    eA = eye(*A.shape,**{'typecode':t})
    trm = mat(eA)
    castfunc = cast[t]
    for k in range(1,q):
        trm *= A / castfunc(k)
        eA += trm
    return eA

_array_precision = {'i': 1, 'l': 1, 'f': 0, 'd': 1, 'F': 0, 'D': 1}
def toreal(arr,tol=None):
    """Return as real array if imaginary part is small.
    """
    if tol is None:
        tol = {0:feps*1e3, 1:eps*1e6}[_array_precision[arr.typecode()]]
    if (arr.typecode() in ['F', 'D']) and \
       sb.allclose(arr.imag, 0.0, atol=tol):
        arr = arr.real
    return arr
 
def cosm(A):
    """matrix cosine.
    """
    A = asarray(A)
    if A.typecode() not in ['F','D']:
        return toreal(0.5*(expm(1j*A) + expm(-1j*A)))
    else:
        return 0.5*(expm(1j*A) + expm(-1j*A))
        
            
def sinm(A):
    """matrix sine.
    """
    A = asarray(A)
    if A.typecode() not in ['F','D']:
        return toreal(-0.5j*(expm(1j*A) - expm(-1j*A)))
    else:
        return -0.5j*(expm(1j*A) - expm(-1j*A))
    
def tanm(A): 
    """matrix tangent.
    """
    A = asarray(A)
    if A.typecode() not in ['F','D']:
        return toreal(solve(cosm(A), sinm(A)))
    else:        
        return solve(cosm(A), sinm(A))
 
def coshm(A): 
    """matrix hyperbolic cosine.
    """
    A = asarray(A)
    if A.typecode() not in ['F','D']:
        return toreal(0.5*(expm(A) + expm(-A)))
    else:
        return 0.5*(expm(A) + expm(-A))
 
def sinhm(A):
    """matrix hyperbolic sine.
    """
    A = asarray(A)
    if A.typecode() not in ['F','D']:
        return toreal(0.5*(expm(A) - expm(-A)))
    else:
        return 0.5*(expm(A) - expm(-A))
    
def tanhm(A):
    """matrix hyperbolic tangent.
    """
    A = asarray(A)
    if A.typecode() not in ['F','D']:
        return toreal(solve(coshm(A), sinhm(A)))
    else:
        return solve(coshm(A), sinhm(A))
 
def funm(A,func,disp=1):
    """matrix function for arbitrary callable object func.
    """
    # func should take a vector of arguments (see vectorize if
    #  it needs wrapping.
 
    # Perform Shur decomposition (lapack ?gees)
    A = asarray(A)
    if len(A.shape)!=2:
        raise ValueError, "Non-matrix input to matrix function."    
    if A.typecode() in ['F', 'D']:
        cmplx_type = 1
    else:
        cmplx_type = 0
    T, Z = schur(A)
    T, Z = rsf2csf(T,Z)
    n,n = T.shape
    F = diag(func(diag(T)))  # apply function to diagonal elements
    F = F.astype(T.typecode()) # e.g. when F is real but T is complex

    minden = abs(T[0,0])

    # implement Algorithm 11.1.1 from Golub and Van Loan
    #                 "matrix Computations." 
    for p in range(1,n):
        for i in range(1,n-p+1):
            j = i + p
            s = T[i-1,j-1] * (F[j-1,j-1] - F[i-1,i-1])
            ksl = slice(i,j-1)
            s = s + dot(T[i-1,ksl],F[ksl,j-1]) - dot(F[i-1,ksl],T[ksl,j-1])
            den = T[j-1,j-1] - T[i-1,i-1]
            if den != 0.0:
                s = s / den
            F[i-1,j-1] = s
            minden = min(minden,abs(den))

    F = dot(dot(Z, F),transpose(conjugate(Z)))
    if not cmplx_type:
        F = toreal(F)

    tol = {0:feps, 1:eps}[_array_precision[F.typecode()]]
    if minden == 0.0:
        minden = tol
    err = min(1, max(tol,(tol/minden)*norm(triu(T,1),1)))
    if product(ravel(logical_not(isfinite(F)))):
        err = Inf
    if disp:
        if err > 1000*tol:
            print "Result may be inaccurate, approximate err =", err
        return F
    else:
        return F, err

def logm(A,disp=1):
    """Matrix logarithm, inverse of expm."""
    # Compute using general funm but then use better error estimator and
    #   make one step in improving estimate using a rotation matrix.
    A = mat(asarray(A))
    F, errest = funm(A,log,disp=0)
    errtol = 1000*eps
    # Only iterate if estimate of error is too large.
    if errest >= errtol:
        # Use better approximation of error
        errest = norm(expm(F)-A,1) / norm(A,1)
        if not isfinite(errest) or errest >= errtol:
            N,N = A.shape
            X,Y = ogrid[1:N+1,1:N+1]
            R = mat(orth(eye(N,typecode='d')+X+Y))
            F, dontcare = funm(R*A*R.H,log,disp=0)
            F = R.H*F*R
            if (norm(imag(F),1)<=1000*errtol*norm(F,1)):
                F = mat(real(F))
            E = mat(expm(F))
            temp = mat(solve(E.T,(E-A).T))
            F = F - temp.T
            errest = norm(expm(F)-A,1) / norm(A,1)
    if disp:
        if not isfinite(errest) or errest >= errtol:
            print "Result may be inaccurate, approximate err =", errest
        return F
    else:
        return F, errest

def signm(a,disp=1):
    """matrix sign"""
    def rounded_sign(x):
        rx = real(x)
        if rx.typecode()=='f':
            c =  1e3*feps*amax(x)
        else:
            c =  1e3*eps*amax(x)
        return sign( (absolute(rx) > c) * rx )
    result,errest = funm(a, rounded_sign, disp=0)
    errtol = {0:1e3*feps, 1:1e3*eps}[_array_precision[result.typecode()]]
    if errest < errtol:
        return result

    # Handle signm of defective matrices:

    # See "E.D.Denman and J.Leyva-Ramos, Appl.Math.Comp.,
    # 8:237-250,1981" for how to improve the following (currently a
    # rather naive) iteration process:

    a = asarray(a)
    #a = result # sometimes iteration converges faster but where??

    # Shifting to avoid zero eigenvalues. How to ensure that shifting does
    # not change the spectrum too much?
    vals = svd(a,compute_uv=0)
    max_sv = sb.amax(vals)
    #min_nonzero_sv = vals[(vals>max_sv*errtol).tolist().count(1)-1]
    #c = 0.5/min_nonzero_sv
    c = 0.5/max_sv
    S0 = a + c*sb.identity(a.shape[0])
    prev_errest = errest
    for i in range(100):
        iS0 = inv(S0)
        S0 = 0.5*(S0 + iS0)
        Pp=0.5*(dot(S0,S0)+S0)
        errest = norm(dot(Pp,Pp)-Pp,1)
        if errest < errtol or prev_errest==errest:
            break
        prev_errest = errest
    if disp:
        if not isfinite(errest) or errest >= errtol:
            print "Result may be inaccurate, approximate err =", errest
        return S0
    else:
        return S0, errest

def sqrtm(A,disp=1):
    """Matrix square root

    If disp is non-zero display warning if singular matrix.
    If disp is zero then return residual ||A-X*X||_F / ||A||_F

    Uses algorithm by Nicholas J. Higham
    """
    A = asarray(A)
    if len(A.shape)!=2:
        raise ValueError, "Non-matrix input to matrix function."    
    if A.typecode() in ['F', 'D']:
        cmplx_type = 1
    else:
        cmplx_type = 0
    T, Z = schur(A)
    T, Z = rsf2csf(T,Z)
    n,n = T.shape

    R = sb.zeros((n,n),T.typecode())
    for j in range(n):
        R[j,j] = sqrt(T[j,j])
        for i in range(j-1,-1,-1):
            s = 0
            for k in range(i+1,j):
                s = s + R[i,k]*R[k,j]
            R[i,j] = (T[i,j] - s)/(R[i,i] + R[j,j])

    R, Z = all_mat(R,Z)
    X = (Z * R * Z.H)

    if disp:
        nzeig = sb.any(sb.diag(T)==0)
        if nzeig:
            print "Matrix is singular and may not have a square root."
        return X.A
    else:
        arg2 = norm(X*X - A,'fro')**2 / norm(A,'fro')
        return X.A, arg2

    








"""
Wrapper functions to more user-friendly calling of certain math functions
whose output is different than the input in certain domains of the input.
"""

__all__ = ['sqrt', 'log', 'log2','logn','log10', 'power', 'arccos',
           'arcsin', 'arctanh']

import Numeric

from type_check import isreal, asarray
from function_base import any
import fastumath
from fastumath import *

__all__.extend([key for key in dir(fastumath) \
                if key[0]!='_' and key not in __all__])

def _tocomplex(arr):
    if arr.typecode() in ['f', 's', 'b', '1','w']:
        return arr.astype('F')
    else:
        return arr.astype('D')

def sqrt(x):
    x = asarray(x)
    if isreal(x) and any(x<0):
        x = _tocomplex(x)
    return fastumath.sqrt(x)

def log(x):
    x = asarray(x)
    if isreal(x) and any(x<0):
        x = _tocomplex(x)
    return fastumath.log(x)

def log10(x):
    x = asarray(x)
    if isreal(x) and any(x<0):
        x = _tocomplex(x)
    return fastumath.log10(x)    

def logn(n,x):
    """ Take log base n of x.
    """
    x = asarray(x)
    if isreal(x) and any(x<0):
        x = _tocomplex(x)
    if isreal(n) and (n<0):
        n = _tocomplex(n)
    return fastumath.log(x)/fastumath.log(n)

def log2(x):
    """ Take log base 2 of x.
    """
    x = asarray(x)
    if isreal(x) and any(x<0):
        x = _tocomplex(x)
    return fastumath.log(x)/fastumath.log(2)


def power(x, p):
    x = asarray(x)
    if isreal(x) and any(x<0):
        x = _tocomplex(x)
    return fastumath.power(x, p)
    
def arccos(x):
    x = asarray(x)
    if isreal(x) and any(abs(x)>1):
        x = _tocomplex(x)
    return fastumath.arccos(x)

def arcsin(x):
    x = asarray(x)
    if isreal(x) and any(abs(x)>1):
        x = _tocomplex(x)
    return fastumath.arcsin(x)

def arctanh(x):
    x = asarray(x)
    if isreal(x) and any(abs(x)>1):
        x = _tocomplex(x)
    return fastumath.arctanh(x)



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# The following is so I know about arrays:
from Numeric import *
from scipy_base.fastumath import *
from shapetest import *
from graftypes import *

class Polymap :
   """This class is for Gist and possibly other graphics packages;
   it creates a class consisting of filled polygons, which can then
   be given to and plotted by a graph2d object.

   x = Polymap ( <keyword arguments> ) will create a Polymap object
   with coordinates and various other characteristics. The keyword
   arguments are:
      x = <sequence of floating point values>
      y = <sequence of floating point values>
          These are the coordinates of the vertices of the
          polygons. The way this data is sewt up, vertices of
          adjacent polygons will be repeated.
      n = <sequence of integer values>
          The entries in this array tell you how many vertices
          each polygon has. The first n[0] entries in x and y define
          the first polygon, the next n[1] entries define the second,
          etc. The sum of all the entried in n is the length of
          vectors x and y.
      z = <sequence of numerical or character values> (this vector
          is the same length as n) tells how to denote the polygons.
          numbers are interpolated into a palette, characters are used
          as is.
      hide = 0/1 (1 to hide this part of the graph)
      label = <string> label for this part of the graph.
   """

   def type (self) :
       return PolyMapType

   _PolyMapSpecError = "PolyMapSpecError"

   def __init__ ( self , * kwds , ** keywords ) :
       if len (kwds) == 1 :
          keywords = kwds[0]
       if not keywords.has_key ("x") or not keywords.has_key ("y") or \
          not keywords.has_key ("z") or not keywords.has_key ("n" ) :
          raise self._PolyMapSpecError , \
             "You need all four keywords x, y, z, and n."
       self.x = keywords ["x"]
       self.y = keywords ["y"]
       self.z = keywords ["z"]
       self.n = keywords ["n"]
       total_size = 0
       if len (self.z) != len (self.n) :
          raise self._PolyMapSpecError , \
             "z and n are not the same size (" + `len (self.z)` + " vs " + \
              `len (self.n)` + ")."
       for i in range ( len(self.n) ) :
          total_size = total_size + self.n [i]
       if len (self.x) != total_size or len (self.y) != total_size :
          raise self._PolyMapSpecError , \
             "Either x or y or both are not the right size (" + \
             `total_size` + ")."
       if keywords.has_key ("hide") :
          self.hide = keywords ["hide"]
       else:
          self.hide = 0
       if keywords.has_key ("label") :
          self.label = keywords ["label"]
       else:
          self.label = " "

   def new ( self, ** keywords ) :
       """ new (...keyword arguments...) allows you to reuse a
       previously existing curve.
       """
       self.__init__ ( keywords )

   def set ( self , ** keywords ) :
       """ set (...keyword arguments...) allows you to set individual
       curve characteristics. No error checking is done.
       """
       for k in keywords.keys ():
           if k == "type" :
              self.line_type = keywords ["type"]
           else :
              setattr (self, k, keywords [k])
